from AccessControl import ClassSecurityInfo
from ftw.shop.config import CATEGORY_RELATIONSHIP
from Persistence import PersistentMapping
from plone.uuid.interfaces import IUUID


class Categorizeable(object):
    """Mixin class for categorizeable objects
    """

    security = ClassSecurityInfo()
    defaultRank = 100

    security.declareProtected("View", 'getRankForCategory')

    def getRankForCategory(self, category):
        """Get the object's rank for the specified category
        """
        if hasattr(self, '_categoryRanks'):
            category_uuid = IUUID(category)
            return int(self._categoryRanks.get(category_uuid, self.defaultRank))
        return self.defaultRank


    security.declareProtected("Modify portal content", 'addToCategory')

    def setRankForCategory(self, category, rank):
        """Set the object's rank for the specified category
        """
        if not hasattr(self, '_categoryRanks') \
            or not isinstance(self._categoryRanks, PersistentMapping):
            self._categoryRanks = PersistentMapping()
        category_uuid = IUUID(category)
        self._categoryRanks[category_uuid] = rank


    security.declareProtected("Modify portal content", 'addToCategory')

    def addToCategory(self, category):
        """Add the object to a category.
        """
        self.addReference(category, CATEGORY_RELATIONSHIP)


    security.declareProtected("Modify portal content", 'removeFromCategory')

    def removeFromCategory(self, category):
        """Remove the object from a category.
        """
        self.deleteReference(category, CATEGORY_RELATIONSHIP)


    security.declareProtected("View", 'listCategories')

    def listCategories(self):
        """List the categories where the object is registered.
        """
        return self.getRefs(CATEGORY_RELATIONSHIP)



from ftw.builder import Builder
from ftw.builder import builder_registry
from ftw.builder import create
from ftw.builder.archetypes import ArchetypesBuilder
from ftw.shop.interfaces import IShopCategory
from ftw.shop.portlets import cart
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletManager
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.hooks import getSite
import transaction


class ShopItemBuilder(ArchetypesBuilder):

    portal_type = 'ShopItem'

    def before_create(self):
        super(ShopItemBuilder, self).before_create()
        if not IShopCategory.providedBy(self.container):
            self.container = create(Builder('shop category'))

builder_registry.register('shop item', ShopItemBuilder)


class ShopCategoryBuilder(ArchetypesBuilder):

    portal_type = 'ShopCategory'

builder_registry.register('shop category', ShopCategoryBuilder)


class ShopSupplierBuilder(ArchetypesBuilder):

    portal_type = 'Supplier'

builder_registry.register('supplier', ShopSupplierBuilder)


class CartPortletBuilder(object):

    def __init__(self, session):
        self.session = session
        self.container = getSite()
        self.arguments = {}
        self.manager_name = u'plone.rightcolumn'
        self.assignment_class = cart.Assignment

    def within(self, container):
        self.container = container
        return self

    def having(self, **kwargs):
        self.arguments.update(kwargs)
        return self

    def create(self):
        self.before_create()
        obj = self.create_portlet()
        self.after_create(obj)
        return obj

    def before_create(self):
        pass

    def create_portlet(self):
        manager = getUtility(IPortletManager,
                             name=self.manager_name,
                             context=self.container)
        assignments = getMultiAdapter((self.container, manager),
                                      IPortletAssignmentMapping,
                                      context=self.container)
        name = self.makeup_name(assignments)
        portlet = self.assignment_class(**self.arguments)
        assignments[name] = portlet
        return portlet

    def makeup_name(self, assignments):
        name = 'cartportlet'

        index = 1
        while name in assignments:
            name = 'cartportlet-%i' % index
            index += 1
        return name

    def after_create(self, obj):
        if self.session.auto_commit:
            transaction.commit()


builder_registry.register('cart portlet', CartPortletBuilder)



from ftw.builder import Builder
from ftw.builder import create
from ftw.shop.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestVariationIndexing(FunctionalTestCase):

    def setUp(self):
        super(TestVariationIndexing, self).setUp()
        self.grant('Manager')

        self.folder = create(Builder('folder'))
        self.category = create(Builder('shop category').within(self.folder))
        self.shopitem = create(Builder('shop item')
                               .titled('Raindrops')
                               .having(variation2_attribute="Farbe",
                                       variation1_values=('Rot', 'Blau'))
                               .within(self.category))

    @browsing
    def test_variations_are_searchable(self, browser):
        browser.visit()
        # check that we don't get any results by default
        browser.fill({'SearchableText': 'shouldnotfind'}).submit()
        self.assertEquals(0, len(browser.css('.searchResults dt a')))

        # check that we find our shopitem with the indexed color.
        browser.fill({'SearchableText': 'rot'}).submit()
        self.assertEquals(1, len(browser.css('.searchResults dt a')))
        self.assertEquals('Raindrops',
                          browser.css('.searchResults dt a').first.text)



from ftw.testbrowser import browser


def is_visible():
    return len(browser.css('.portletCartPortlet')) > 0


def is_empty():
    empty = browser.css('.portletCartPortlet .cart-empty')
    if empty:
        return empty.first.text
    else:
        return False


def items():
    return browser.css('.portletCartPortlet .cart-items li a').text


def edit_cart_link():
    actions = browser.css('.portletCartPortlet .cartActions').first_or_none
    return actions and actions.find('Edit cart')


def order_link():
    actions = browser.css('.portletCartPortlet .cartActions').first_or_none
    return actions and actions.find('Order')


def order_manager_link():
    return browser.css('.portletCartPortlet').first.find('Order Manager')



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from Testing.ZopeTestCase.utils import setupCoreSessions
from zope.configuration import xmlconfig
import ftw.shop.tests.builders


class FtwShopLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)
        setupCoreSessions(app)
        z2.installProduct(app, 'ftw.shop')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.shop:default')


FTW_SHOP_FIXTURE = FtwShopLayer()
FTW_SHOP_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_SHOP_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.shop:functional")

FTW_SHOP_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_SHOP_FIXTURE, ), name="ftw.shop:integration")



import base64
import simplejson
from datetime import datetime, timedelta

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from collective.z3cform.wizard import wizard
from plone.registry.interfaces import IRegistry
from plone.z3cform.layout import FormWrapper
from z3c.form import field, button
from zope.interface import implements, Interface
from zope.component import getMultiAdapter, adapts
from zope.component import getAdapters
from zope.component import getUtility

from ftw.shop import shopMessageFactory as _
from ftw.shop.browser.widgets.paymentprocessor import PaymentProcessorFieldWidget
from ftw.shop.config import COOKIE_ADDRESS_KEY
from ftw.shop.config import SESSION_ADDRESS_KEY
from ftw.shop.config import SESSION_REVIEW_KEY
from ftw.shop.config import SESSION_SHIPPING_KEY
from ftw.shop.interfaces import IContactInformationStep
from ftw.shop.interfaces import IContactInformationStepGroup
from ftw.shop.interfaces import IDefaultContactInformation
from ftw.shop.interfaces import IDefaultPaymentProcessorChoice
from ftw.shop.interfaces import IOrderReviewSchema
from ftw.shop.interfaces import IOrderReviewStep
from ftw.shop.interfaces import IOrderReviewStepGroup
from ftw.shop.interfaces import IPaymentProcessor
from ftw.shop.interfaces import IPaymentProcessorChoiceStep
from ftw.shop.interfaces import IPaymentProcessorStepGroup
from ftw.shop.interfaces import IShippingAddress
from ftw.shop.interfaces import IShippingAddressStep
from ftw.shop.interfaces import IShippingAddressStepGroup
from ftw.shop.interfaces import IShopConfiguration


try:
    # Plone >= 4.1
    from zope.browserpage import viewpagetemplatefile as zvptf

except ImportError:
    # Plone < 4.1
    from zope.app.pagetemplate import viewpagetemplatefile as zvptf



class BaseStepGroup(object):

    def __init__(self, context, request, dummy):
        pass


class BasePaymentProcessor(object):

    def __init__(self, context, request, dummy):
        pass

    def available(self):
        return True


class DefaultContactInfoStep(wizard.Step):
    implements(IContactInformationStep)
    prefix = 'contact_information'
    label = _(u"label_default_contact_info_step",
              default="Contact Information")
    title = _(u"title_default_contact_info_step",
              default="Default Contact Information")
    description = _(u'help_default_contact_info_step', default=u"")
    fields = field.Fields(IDefaultContactInformation)

    prefill_fields = ['title', 'firstname', 'lastname', 'email',
                      'company', 'street1', 'street2', 'phone',
                      'zipcode', 'city', 'country']

    def __init__(self, context, request, wiz):
        super(wizard.Step, self).__init__(context, request)
        self.wizard = wiz

    def updateWidgets(self):
        super(DefaultContactInfoStep, self).updateWidgets()
        request = self.context.REQUEST

        # if user is authenticated:
        # prefill form from portal_memberdata
        mt = getToolByName(self.context, 'portal_membership')
        if not mt.isAnonymousUser():
            member = mt.getAuthenticatedMember()
            fullname = member.getProperty('fullname', '')

            if not isinstance(fullname, unicode):
                # assume it's uft-8
                fullname = fullname.decode('utf-8')

            if fullname.find(' ') > -1:
                firstname, lastname = fullname.rsplit(' ', 1)
            else:
                firstname = lastname = ''
            email = member.getProperty('email')

            self.widgets['firstname'].value = firstname
            self.widgets['lastname'].value = lastname
            self.widgets['email'].value = unicode(email)


        if COOKIE_ADDRESS_KEY in request:
            # Prefill contact data with values from cookie
            cookie_data = simplejson.loads(base64.b64decode(
                                    request[COOKIE_ADDRESS_KEY]))
            for key in cookie_data.keys():
                if isinstance(cookie_data[key], basestring):
                    cookie_data[key] = unicode(cookie_data[key])

            for fieldname in self.prefill_fields:
                try:
                    self.widgets[fieldname].value = cookie_data[fieldname]
                except KeyError:
                    pass


        elif SESSION_ADDRESS_KEY in request.SESSION.keys():
            # Prefill contact data form with values from session
            contact_info = request.SESSION[SESSION_ADDRESS_KEY]
            for fieldname in self.prefill_fields:
                try:
                    self.widgets[fieldname].value = contact_info[fieldname]
                except KeyError:
                    pass

        self.widgets['zipcode'].size = 5


class DefaultContactInfoStepGroup(BaseStepGroup):
    implements(IContactInformationStepGroup)
    adapts(Interface, Interface, Interface)
    title = _(u"title_default_contact_info_step_group",
              default="Default Contact Information")
    steps = (DefaultContactInfoStep, )


class DefaultShippingAddressStep(wizard.Step):
    implements(IShippingAddressStep)
    prefix = 'shipping_address'
    label = _(u"label_default_shipping_address_step",
              default="Shipping Address")
    title = _(u"title_default_shipping_address_step",
              default="Default Shipping Address")
    description = _(u'help_default_shipping_address_step', default=u"")
    prefill_fields = [
        'title',
        'firstname',
        'lastname',
        'company',
        'street1',
        'street2',
        'zipcode',
        'city']
    fields = field.Fields(IShippingAddress) + \
    field.Fields(IDefaultContactInformation).select(
        'title',
        'firstname',
        'lastname',
        'company',
        'street1',
        'street2',
        'zipcode',
        'city')

    def __init__(self, context, request, wiz):
        super(wizard.Step, self).__init__(context, request)
        self.wizard = wiz

    def updateWidgets(self):
        super(DefaultShippingAddressStep, self).updateWidgets()
        if 'contact_information' in self.wizard.session and \
            (not 'shipping_address' in self.wizard.session \
            or self.wizard.session['shipping_address'] == {}):
            # Prefill shipping address with data from contact_info (prev step)
            contact_info = self.wizard.session['contact_information']
            for fieldname in self.prefill_fields:
                try:
                    self.widgets[fieldname].value = contact_info[fieldname]
                except KeyError:
                    pass


class DefaultShippingAddressStepGroup(BaseStepGroup):
    implements(IShippingAddressStepGroup)
    adapts(Interface, Interface, Interface)
    title = _(u"title_default_shipping_address_step_group",
              default="Default Shipping Address")
    steps = (DefaultShippingAddressStep, )


class DefaultPaymentProcessorChoiceStep(wizard.Step):
    implements(IPaymentProcessorChoiceStep)
    prefix = 'payment_processor_choice'
    label = _(u"label_default_payment_processor_choice_step",
              default="Payment Processor")
    title = _(u"title_default_payment_processor_step",
              default="Default Payment Processor Choice")
    description = _(u'help_default_payment_processor_choice_step',
                    default=u"")
    fields = field.Fields(IDefaultPaymentProcessorChoice)
    fields['payment_processor'].widgetFactory = PaymentProcessorFieldWidget


class DefaultPaymentProcessorStepGroup(BaseStepGroup):
    implements(IPaymentProcessorStepGroup)
    adapts(Interface, Interface, Interface)
    title = _(u"title_default_payment_processor_step_group",
              default="Default Payment Processor Choice")
    steps = (DefaultPaymentProcessorChoiceStep, )


class InvoicePaymentProcessor(BasePaymentProcessor):
    implements(IPaymentProcessor)
    adapts(Interface, Interface, Interface)

    external = False
    url = None
    title = "Gegen Rechnung"
    image = """<img src="++resource++ftw-shop-resources/einzahlungsschein.png" />"""
    description = """<em>Bezahlung gegen Rechnung</em>"""

    def available(self):
        return True


class DefaultOrderReviewStep(wizard.Step):
    implements(IOrderReviewStep)
    prefix = 'order-review'
    label = _(u'label_order_review_step', default="Order Review")
    title = _(u"title_default_order_review_step",
              default="Default Order Review Step")
    description = _(u'help_order_review_step', default=u'')
    index = zvptf.ViewPageTemplateFile('templates/checkout/order_review.pt')
    fields = field.Fields(IOrderReviewSchema)

    def __init__(self, context, request, wiz):
        super(wizard.Step, self).__init__(context, request)
        self.wizard = wiz


class DefaultOrderReviewStepGroup(BaseStepGroup):
    implements(IOrderReviewStepGroup)
    adapts(Interface, Interface, Interface)
    title = _(u"title_default_order_review_step_group",
              default="Default Order Review Step Group")
    steps = (DefaultOrderReviewStep, )


class CheckoutWizard(wizard.Wizard):
    label = _(u"label_checkout_wizard", default="Checkout")
    index = zvptf.ViewPageTemplateFile('templates/checkout-wizard.pt')

    def __init__(self, context, request):
        super(CheckoutWizard, self).__init__(context, request)
        self.context = context
        self.request = request

    def getSelectedPaymentProcessor(self):
        payment_processor = None
        if 'payment_processor_choice' not in self.session.keys():
            return None
        try:
            pp_name = self.session.get(
                    'payment_processor_choice').get('payment_processor')
        except KeyError:
            try:
                pp_name = self.request.SESSION.get(
                        'payment_processor_choice').get('payment_processor')
            except KeyError:
                # No payment processor step activated
                pp_name = "none"
        for name, adapter in getAdapters((self.context, self.request, self.context),
                                         IPaymentProcessor):
            if name == pp_name:
                payment_processor = adapter
        return payment_processor

    @property
    def steps(self):
        contact_info_steps = ()
        shipping_address_steps = ()
        contact_info_step_groups = getAdapters(
                                        (self.context, self.request, self),
                                        IContactInformationStepGroup)
        shipping_address_step_groups = getAdapters(
                                        (self.context, self.request, self),
                                        IShippingAddressStepGroup)
        payment_processor_step_groups = getAdapters(
                                        (self.context, self.request, self),
                                        IPaymentProcessorStepGroup)
        order_review_step_groups = getAdapters(
                                        (self.context, self.request, self),
                                        IOrderReviewStepGroup)
        registry = getUtility(IRegistry)
        shop_config = registry.forInterface(IShopConfiguration)

        # Get steps for selected Contact Info Step Group
        selected_contact_info_step_group = shop_config.contact_info_step_group
        for name, step_group_adapter in contact_info_step_groups:
            if name == selected_contact_info_step_group:
                contact_info_steps = step_group_adapter.steps

        # Get steps for selected Shipping Address Step Group
        selected_shipping_address_step_group = shop_config.shipping_address_step_group
        for name, step_group_adapter in shipping_address_step_groups:
            if name == selected_shipping_address_step_group:
                shipping_address_steps = step_group_adapter.steps

        # Get steps for selected Payment Processor Step Group
        selected_pp_step_group = shop_config.payment_processor_step_group
        for name, step_group_adapter in payment_processor_step_groups:
            if name == selected_pp_step_group:
                payment_processor_steps = step_group_adapter.steps

        # Get steps for selected Order Review Step Group
        selected_order_review_step_group = shop_config.order_review_step_group
        for name, step_group_adapter in order_review_step_groups:
            if name == selected_order_review_step_group:
                order_review_steps = step_group_adapter.steps

        return contact_info_steps + \
               shipping_address_steps + \
               payment_processor_steps + \
               order_review_steps

    @button.buttonAndHandler(_(u'btn_back', default="Back"),
                             name='back',
                             condition=lambda form: not form.onFirstStep)
    def handleBack(self, action):
        data, errors = self.currentStep.extractData()
        if errors:
            errorMessage ='<ul>'
            for error in errors:
                if errorMessage.find(error.message):
                    errorMessage += '<li>' + error.message + '</li>'
            errorMessage += '</ul>'
            self.status = errorMessage

        else:
            self.currentStep.applyChanges(data)
            self.updateCurrentStep(self.currentIndex - 1)

            # Back can change the conditions for the finish button,
            # so we need to reconstruct the button actions, since we
            # do not redirect.
            self.updateActions()

    @button.buttonAndHandler(_(u'btn_continue', default='Next'),
                             name='continue',
                             condition=lambda form: not form.onLastStep)
    def handleContinue(self, action):
        data, errors = self.currentStep.extractData()
        if errors:
            errorMessage ='<ul>'
            for error in errors:
                if errorMessage.find(error.message):
                    errorMessage += '<li>' + error.message + '</li>'
            errorMessage += '</ul>'
            self.status = errorMessage
        else:

            self.currentStep.applyChanges(data)
            self.updateCurrentStep(self.currentIndex + 1)

            # Proceed can change the conditions for the finish button,
            # so we need to reconstruct the button actions, since we
            # do not redirect.
            self.updateActions()

    @button.buttonAndHandler(_(u'btn_finish', default='Finish'),
             name='finish',
             condition=lambda form: form.allStepsFinished or form.onLastStep)
    def handleFinish(self, action):
        data, errors = self.currentStep.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        else:
            self.status = self.successMessage
            self.finished = True
        self.currentStep.applyChanges(data)
        self.finish()

        address_data = {}
        address_data.update(self.session['contact_information'])
        self.request.SESSION[SESSION_ADDRESS_KEY] = address_data

        shipping_data = {}
        shipping_data.update(self.session['shipping_address'])
        self.request.SESSION[SESSION_SHIPPING_KEY] = shipping_data

        review_data = {}
        review_data.update(self.session['order-review'])
        self.request.SESSION[SESSION_REVIEW_KEY] = review_data

        self.request.SESSION['order_confirmation'] = True

        payment_processor_choice = {}
        if 'payment_processor_choice' in self.session.keys():
            payment_processor_choice.update(self.session['payment_processor_choice'])
        self.request.SESSION['payment_processor_choice'] = payment_processor_choice

        # Save contact information in a cookie in order to prefill
        # form if customer returns
        cookie_value = base64.b64encode(simplejson.dumps(
                                        self.session['contact_information']))
        expiry_date = (datetime.now() + timedelta(days=90)).strftime(
                                                "%a, %d-%b-%Y %H:%M:%S GMT")
        self.request.RESPONSE.setCookie(COOKIE_ADDRESS_KEY,
                                        cookie_value,
                                        path='/',
                                        expires=expiry_date)

        self.request.SESSION[self.sessionKey] = {}
        self.sync()

        pp = self.getSelectedPaymentProcessor()
        if pp is not None and pp.external:
            self.request.SESSION['external-processor-url'] = pp.url
            self.request.SESSION['external-processor-url'] = "http://localhost:8077/"

        self.request.response.redirect('checkout')


class CheckoutView(FormWrapper):
    #layout = ViewPageTemplateFile("templates/layout.pt")
    #index = layout
    form = CheckoutWizard

    def __init__(self, context, request):
        cart_view = getMultiAdapter((context, request), name='cart_view')
        request['cart_view'] = cart_view
        FormWrapper.__init__(self, context, request)


class ExternalPaymentProcessorView(BrowserView):
    """Redirects to an external payment processor
    """
    __call__ = ViewPageTemplateFile('templates/checkout/external.pt')



class MissingCustomerInformation(Exception):
    """Gets raised if trying to add an order when customer information has
    not been supplied yet.
    """
    pass


class MissingShippingAddress(Exception):
    """Gets raised if trying to add an order when shipping address has
    not been supplied yet.
    """
    pass


class MissingOrderConfirmation(Exception):
    """Gets raised if trying to add an order but the customer hasn't confirmed
    the order in the review step yet.
    """
    pass


class MissingPaymentProcessor(Exception):
    """Gets raised if trying to add an order but the customer hasn't selected
    a payment processor yet.
    """
    pass



from ftw.shop import portlets
from ftw.shop import shopMessageFactory as _
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletManager
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
from zope.component import getUtility


class InitShopStructure(BrowserView):
    """Set up an initial folder structure for the shop
    """

    def __call__(self):
        portal_url = getToolByName(self.context, "portal_url")
        ptool = getToolByName(self.context, 'plone_utils')

        portal = portal_url.getPortalObject()
        shop = getattr(portal, 'shop', None)
        if not shop:
            portal.invokeFactory('Folder', 'shop', title='Shop')
            shop = portal.shop

        # Add Shopping Cart portlet to Shop Root
        column = getUtility(IPortletManager,
                            name=u'plone.rightcolumn',
                            context=shop)
        manager = getMultiAdapter((shop, column), IPortletAssignmentMapping)
        if 'ftw.shop.portlets.cart' not in manager.keys():
            manager['ftw.shop.portlets.cart'] = portlets.cart.Assignment()

        ptool.addPortalMessage(_(u'msg_shop_initialized',
                                 default=u"Shop structure initialized."),
                               'info')
        # redirect to referer
        referer = self.request.get('HTTP_REFERER', portal.absolute_url())
        if not referer in ['', 'localhost']:
            self.request.response.redirect(referer)
        else:
            self.request.response.redirect(portal.absolute_url())
        return



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.shop import shopMessageFactory as _
from ftw.shop.interfaces import IShopItem
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from zope import schema
from zope.component import getMultiAdapter
from zope.formlib import form
from zope.interface import implements
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


displayChoices = SimpleVocabulary(
    [SimpleTerm(value='always',
                token='always',
                title=_(u'label_always',
                        default=u'Always'),
                ),
     SimpleTerm(value='only_if_items',
                token='only_if_items',
                title=_(u'label_only_if_items',
                        default=u'Only if ShopItems available in current folder'),
                )
      ]
    )


class ICartPortlet(IPortletDataProvider):
    """A portlet that displays the shopping cart
    """
    mode = schema.Choice(title=_(u'Display mode'),
        description=_(u'Conditions under which portlet should be shown'),
        vocabulary=displayChoices,
        default='always'
        )


class Assignment(base.Assignment):
    """Portlet assignment.

    This is what is actually managed through the portlets UI and associated
    with columns.
    """

    implements(ICartPortlet)

    def __init__(self, mode='always'):
        self.mode = mode

    @property
    def title(self):
        """This property is used to give the title of the portlet in the
        "manage portlets" screen.
        """
        return _(u"label_cart_portlet", default=u"Cart Portlet")


class Renderer(base.Renderer):
    """Portlet renderer.

    This is registered in configure.zcml. The referenced page template is
    rendered, and the implicit variable 'view' will refer to an instance
    of this class. Other methods can be added and referenced in the template.
    """

    render = ViewPageTemplateFile('cart.pt')

    @property
    def available(self):
        # If the shopping cart contains items, we ALWAYS display the portlet
        if self.items_in_cart():
            return True

        if self.data.mode == 'only_if_items':
            # Only display portlet if there are shop items in current folder
            if self.context_has_shopitems():
                return True
            else:
                return False
        return True

    def context_has_shopitems(self):
        shopitem_ifaces = ['ftw.shop.interfaces.IShopItem',
                            'ftwshop.simplelayout.interfaces.IShopItemBlock']
        shop_contents = self.context.getFolderContents(
                            contentFilter={'object_provides': shopitem_ifaces})
        if shop_contents or IShopItem.providedBy(self.context):
            return True
        return False

    def items_in_cart(self):
        cart_view = getMultiAdapter((self.context, self.request), name='cart_view')
        return cart_view.cart_items()


class AddForm(base.AddForm):
    form_fields = form.Fields(ICartPortlet)
    label = _(u"label_add_cart_portlet",
              default=u"Add Shopping Cart Portlet")
    description = _(u"help_add_cart_portlet",
                default=u"This portlet displays the shopping cart contents.")

    def create(self, data):
        assignment = Assignment(**data)
        return assignment


class EditForm(base.EditForm):
    form_fields = form.Fields(ICartPortlet)
    label = _(u"Edit Shopping Cart portlet")
    description = _(u"This portlet displays the Shopping Cart.")



from ftw.testbrowser import browser


def add_to_cart(amount=1):
    form = browser.find('Add to cart').form
    form.fill({'quantity:int': str(amount)})
    form.find('Add to cart').click()

def edit_categories_link():
    return browser.css('#contentview-categories').first.find('Categories')



# -*- coding: utf-8 -*-
import cStringIO
import csv
from datetime import datetime
from datetime import date
from email.Utils import formataddr

from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import noSecurityManager
from Acquisition import aq_inner
from plone.registry.interfaces import IRegistry
from Products.Archetypes.utils import contentDispositionHeader
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getAdapter
from zope.component import getAdapters
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import implements
from zope.publisher.interfaces import IPublishTraverse
from zope.publisher.interfaces.browser import IBrowserView
from zope.i18n import translate

from ftw.shop import shopMessageFactory as _
from ftw.shop.config import ONLINE_PENDING_KEY
from ftw.shop.config import SESSION_ADDRESS_KEY
from ftw.shop.config import SESSION_REVIEW_KEY
from ftw.shop.config import SESSION_SHIPPING_KEY
from ftw.shop.exceptions import MissingCustomerInformation
from ftw.shop.exceptions import MissingOrderConfirmation
from ftw.shop.exceptions import MissingPaymentProcessor
from ftw.shop.exceptions import MissingShippingAddress
from ftw.shop.interfaces import IMailHostAdapter
from ftw.shop.interfaces import IOrderStorage
from ftw.shop.interfaces import IPaymentProcessorStepGroup
from ftw.shop.interfaces import IShopConfiguration

try:
    # Python > 2.5
    strptime = datetime.strptime
except AttributeError:
    # Python 2.4
    import time
    def strptime(date_string, format):
        return datetime(*(time.strptime(date_string, format)[0:6]))

try:
    dummy = type(any)
except NameError:
    # Python 2.4
    def any(iterable):
        for element in iterable:
            if element:
                return True
        return False



DEBUG = False

COLUMN_TITLES = {
    'order_id': 'Bestellungs-ID',
    'title': 'Bestellnummer',
    'status': 'Status',
    'total': 'Total',
    'date': 'Datum',
    'customer_attn': 'Zu Handen von',
    'customer_title': 'Anrede',
    'customer_firstname': 'Vorname',
    'customer_lastname': 'Nachname',
    'customer_company': 'Firma',
    'customer_department_code': 'Postkürzel',
    'customer_cost_center': 'Kostenstelle',
    'customer_email': 'e-Mail',
    'customer_street1': 'Strasse',
    'customer_street2': 'Adresszusatz',
    'customer_phone': 'Telefon',
    'customer_zipcode': 'PLZ',
    'customer_city': 'Ort',
    'customer_shipping_address': 'Bestelladresse',
    'customer_country': 'Land',
    'customer_comments': 'Kommentare',

    'shipping_title': 'Lieferadresse Anrede',
    'shipping_firstname': 'Lieferadresse Vorname',
    'shipping_lastname': 'Lieferadresse Nachname',
    'shipping_company': 'Lieferadresse Firma',
    'shipping_department_code': 'Lieferadresse Postkürzel',
    'shipping_street1': 'Lieferadresse Strasse',
    'shipping_street2': 'Lieferadresse Adresszusatz',
    'shipping_zipcode': 'Lieferadresse PLZ',
    'shipping_city': 'Lieferadresse Ort',

    'sku_code': 'Artikel-Nr',
    'quantity': 'Menge',
    'dimensions': 'Abmessung',
    'title': 'Titel',
    'price': 'Preis',
    'show_price': 'Preis anzeigen',
    'item_total': 'Subtotal',
    'supplier_name': 'Lieferant',
    'supplier_email': 'Lieferant e-Mail',
    'vat_amount': 'MwSt',
}


class OrderManagerView(BrowserView):
    """Lists orders stored in a IOrderStorage
    """

    template = ViewPageTemplateFile('templates/order_manager.pt')
    change_status_tmpl = ViewPageTemplateFile('templates/status.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.request.set('disable_border', True)
        registry = getUtility(IRegistry)
        self.shop_config = registry.forInterface(IShopConfiguration)
        self.mhost = IMailHostAdapter(self.context)
        self.order_storage = getUtility(IOrderStorage,
                                        name=self.shop_config.order_storage)
        self.supplier_filter = None
        self.status_filter = None
        super(OrderManagerView, self).__init__(context, request)


    def __call__(self):
        from_date, to_date = self.parse_date_range()
        self.supplier_filter = self.request.form.get('supplier',
            'all_suppliers')
        self.status_filter = self.request.form.get('form.widgets.status_filter', 'all')
        self.order_results = self.getOrders(from_date, to_date)
        if self.request.form.get('filter'):
            return self.__of__(self.context).template()
        elif self.request.form.get('download_csv'):
            return self.download_csv()
        elif self.request.form.get('cancel_orders'):
            return self.cancel_orders()
        elif self.request.form.get('change_status_step1'):
            return self.__of__(self.context).change_status_tmpl()
        elif self.request.form.get('change_status'):
            return self.change_status()
        else:
            return self.__of__(self.context).template()


    def parse_date_range(self):
        """Parse the from_date and to_date strings,
        assuming the jQuery UI datepicker widget always
        stores them in the request in the "%d.%m.%Y"
        format.
        """

        try:
            from_date = strptime(self.request.form.get(
                'from_date', ''), "%d.%m.%Y").date()
        except ValueError:
            from_date = date(2001, 1, 1)
        try:
            to_date = strptime(self.request.form.get(
                'to_date', ''), "%d.%m.%Y").date()
        except ValueError:
            to_date = date(2100, 1, 1)
        return (from_date, to_date)


    def getSuppliers(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        brains = catalog(portal_type="Supplier")
        results = [dict(name=b.Title, email=b.email) for b in brains]
        return results

    def download_csv(self):
        """Returns a CSV file containing the shop orders
        """

        filename = "orders.csv"
        stream = cStringIO.StringIO()
        csv_writer = csv.writer(stream, dialect='excel', delimiter=';',
                                quotechar='"', quoting=csv.QUOTE_MINIMAL)
        core_cols = ['order_id',
                      'title',
                      'status',
                      'total',
                      'date',
                      'customer_title',
                      'customer_firstname',
                      'customer_lastname',
                      'customer_company',
                      'customer_email',
                      'customer_street1',
                      'customer_street2',
                      'customer_phone',
                      'customer_zipcode',
                      'customer_city',
                      'customer_shipping_address',
                      'customer_country',
                      'customer_comments']

        # Create union of core_cols + all_cols to retain order
        all_cols = self.order_storage.getFieldNames()
        columns = core_cols + filter(lambda x:x not in core_cols, all_cols)
        cart_cols = ['sku_code', 'quantity', 'dimensions', 'title', 'price',
                     'item_total', 'supplier_name', 'supplier_email']

        column_titles = [COLUMN_TITLES[col].decode('utf-8').encode('cp1252') for col in columns + cart_cols]
        # Write header row
        csv_writer.writerow(column_titles)

        for order in self.order_results:
            order_data = [getattr(order, attr, '') for attr in columns]

            # Get the total via getTotal accessor to convert it to Decimal
            order_data[columns.index('total')] = order.getTotal()

            for i, value in enumerate(order_data):
                if isinstance(value, unicode):
                    order_data[i] = value.encode('cp1252')

            for cart_item in order.cartitems:
                # format dimensions
                dimensions = [(cart_item.dimensions[i], dim) for i, dim in enumerate(cart_item.selectable_dimensions)]
                dim_strings = [' '.join([str(dim[0]), translate(_(dim[1]), context=self.request)]) for dim in dimensions]
                dim_formatted = ','.join(dim_strings)

                cart_data = [cart_item.sku_code,
                            cart_item.quantity,
                            dim_formatted,
                            cart_item.title,
                            cart_item.getPrice(),
                            cart_item.getTotal(),
                            cart_item.supplier_name,
                            cart_item.supplier_email]

                for i, value in enumerate(cart_data):
                    if isinstance(value, unicode):
                        cart_data[i] = value.encode('cp1252')

                csv_writer.writerow(order_data + cart_data)

        RESPONSE = self.request.RESPONSE
        header_value = contentDispositionHeader('attachment',
                                                'cp1252',
                                                filename=filename)
        if not DEBUG:
            RESPONSE.setHeader("Content-Disposition", header_value)
            RESPONSE.setHeader("Content-Type",
                           'text/comma-separated-values;charset=%s' % 'cp1252')
        else:
            RESPONSE.setHeader("Content-Type",
                           'text/plain; charset=%s' % 'cp1252')

        stream.seek(0)
        return stream.read()

    def cancel_orders(self):
        """Cancel orders by their order_ids
        """
        context = aq_inner(self.context)
        ptool = getToolByName(context, 'plone_utils')
        order_storage = self.getOrderStorage()

        orders = self.request.form.get('orders', [])
        for order_id in orders:
            try:
                order_id = int(order_id)
            except ValueError:
                # Invalid order_id - continue with next one
                continue
            order_storage.cancelOrder(order_id)
        msg = _(u'msg_order_cancelled',
                default=u"${no_orders} orders cancelled.",
                mapping={ u"no_orders" : len(orders)})
        ptool.addPortalMessage(msg, 'info')
        self.request.response.redirect("%s/order_manager" % context.absolute_url())

    def change_status(self):
        """Change status of several orders at once
        """
        context = aq_inner(self.context)
        ptool = getToolByName(context, 'plone_utils')
        order_storage = self.getOrderStorage()

        orders = self.request.form.get('orders', [])
        new_status = self.request.form.get('form.widgets.status', None)

        # Change status of the orders
        for order_id in orders:
            try:
                order_id = int(order_id)
            except ValueError:
                # Invalid order_id - continue with next one
                continue
            order = order_storage.getOrder(order_id)
            order.status = int(new_status)

        msg = _(u'msg_status_changed',
                default=u"Changed status for ${no_orders} orders.",
                mapping={ u"no_orders" : len(orders)})
        ptool.addPortalMessage(msg, 'info')
        return self.template()


    def getOrders(self, from_date=None, to_date=None):
        order_storage = self.order_storage
        all_orders = order_storage.getAllOrders()

        if any([from_date and to_date, self.supplier_filter, self.status_filter]):
            # Filter orders
            orders = []
            for order in all_orders:
                suppliers = [item.supplier_name for item in order.cartitems]
                if (self.supplier_filter in suppliers
                or self.supplier_filter in ["all_suppliers", "", None]) \
                and self.status_filter == str(order.status) \
                or self.status_filter == "all" \
                and from_date <= order.date.date() \
                and to_date >= order.date.date():
                    orders.append(order)
        else:
            orders = all_orders
        return orders

    def getOrder(self, order_id):
        order_storage = self.order_storage
        order = order_storage.getOrder(order_id)
        return order

    def getOrderStorage(self):
        return self.order_storage

    def addOrder(self):
        """Add a new Order and return the order id.
        """

        session = self.context.REQUEST.SESSION

        # check for cart
        cart_view = getMultiAdapter((self.context, self.context.REQUEST),
                                    name=u'cart_view')
        cart_data = cart_view.cart_items()

        # check for customer data
        customer_data = session.get(SESSION_ADDRESS_KEY, {})
        if not customer_data:
            raise MissingCustomerInformation

        # check for shipping address
        shipping_data = session.get(SESSION_SHIPPING_KEY, {})
        if not shipping_data:
            raise MissingShippingAddress

        # check for review data
        review_data = session.get(SESSION_REVIEW_KEY, {})
        # The comment was previously in the customer data step. If we move it
        # to the customer data set we can avoid changing all templates.
        customer_data.update(review_data)

        # check for order confirmation
        if not session.get('order_confirmation', None):
            raise MissingOrderConfirmation

        # check for payment processor
        payment_processor_step_groups = getAdapters(
                                        (self.context, self.request, self),
                                        IPaymentProcessorStepGroup)

        selected_pp_step_group = self.shop_config.payment_processor_step_group
        for name, step_group_adapter in payment_processor_step_groups:
            if name == selected_pp_step_group:
                payment_processor_steps = step_group_adapter.steps

        if not len(payment_processor_steps) == 0 \
            and not session.get('payment_processor_choice', None):
            raise MissingPaymentProcessor

        # change security context to owner
        user = self.context.getWrappedOwner()
        newSecurityManager(self.context.REQUEST, user)

        order_storage = self.order_storage
        order_id = order_storage.createOrder(status=ONLINE_PENDING_KEY,
                                             date=datetime.now(),
                                             customer_data=customer_data,
                                             shipping_data=shipping_data,
                                             total=cart_view.cart_total(),
                                             cart_data=cart_data)
        order_storage.flush()

        noSecurityManager()

        return order_id

    def sendOrderMails(self, order_id):
        """Send order confirmation and notification mails for the order with
        the specified order_id.
        """

        order = self.order_storage.getOrder(order_id)

        ltool = getToolByName(self.context, 'portal_languages')
        lang = ltool.getPreferredLanguage()

        # Send order confirmation to customer
        self._send_customer_mail(order, lang)

        # Send order notification to supplier(s)
        suppliers = []
        for item_type in order.cartitems:
            if not (item_type.supplier_name == '' \
                or item_type.supplier_email == ''):
                for email_address in item_type.supplier_email.split(','):
                    suppliers.append((item_type.supplier_name,
                                      email_address.strip()))
        unique_suppliers = set(suppliers)
        for supplier in unique_suppliers:
            self._send_supplier_mail(supplier, order)

        # Send order notification to shop owner
        if self.shop_config.always_notify_shop_owner:
            self._send_owner_mail(order)

        return

    def _send_owner_mail(self, order):
        """Send order notification to shop owner.
        """
        show_prices = self.show_prices(order)
        mail_to = formataddr(("Shop Owner", self.shop_config.shop_email))
        customer_name = "%s %s" % (order.customer_firstname,
                                   order.customer_lastname)
        customer_address = formataddr((customer_name, order.customer_email))
        mail_subject = '[%s] Order %s by %s' % (self.shop_config.shop_name,
                                               order.order_id,
                                               customer_name)

        mail_view = getMultiAdapter((self.context, self.context.REQUEST),
                                    name=u'shopowner_mail_view')
        msg_body = mail_view(order=order,
                             show_prices=show_prices,
                             has_order_dimensions=self.has_order_dimensions(order),
                             shop_config=self.shop_config)
        self._send_mail(mail_to, mail_subject, msg_body, reply_to=customer_address)

    def _send_customer_mail(self, order, lang):
        """Send order confirmation mail to customer
        """
        show_prices = self.show_prices(order)
        customer_name = "%s %s" % (order.customer_firstname,
                                   order.customer_lastname)
        mail_to = formataddr((customer_name, order.customer_email))

        mail_subject = self.shop_config.mail_subject
        if not mail_subject:
            mail_subject = u'Your Webshop Order'
        mail_subject = translate(mail_subject, domain='ftw.shop',
                                 context=self.request, default=mail_subject)

        mail_view = getMultiAdapter((self.context, self.context.REQUEST),
                                    name=u'mail_view')
        msg_body = mail_view(order=order,
                             show_prices=show_prices,
                             has_order_dimensions=self.has_order_dimensions(order),
                             shop_config=self.shop_config)
        self._send_mail(mail_to, mail_subject, msg_body)

    def _send_supplier_mail(self, supplier, order):
        """Send order notification to a (single) supplier.
        """
        show_prices = self.show_prices(order)
        mail_to = formataddr(supplier)
        customer_name = "%s %s" % (order.customer_firstname,
                                   order.customer_lastname)
        customer_address = formataddr((customer_name, order.customer_email))
        mail_subject = '[%s] Order %s by %s' % (self.shop_config.shop_name,
                                               order.order_id,
                                               customer_name)

        mail_view = getMultiAdapter((self.context, self.context.REQUEST),
                                    name=u'supplier_mail_view')
        msg_body = mail_view(order=order,
                             show_prices=show_prices,
                             has_order_dimensions=self.has_order_dimensions(order),
                             shop_config=self.shop_config,
                             supplier=supplier)
        self._send_mail(mail_to, mail_subject, msg_body, reply_to=customer_address)

    def _send_mail(self, to, subject, body, reply_to=None):
        """Send mail originating from the shop.
        """

        mail_bcc = getattr(self.shop_config, 'mail_bcc', '')
        mail_from = formataddr((self.shop_config.shop_name,
                               self.shop_config.shop_email))
        self.mhost.send(body,
             mto=to,
             mfrom=mail_from,
             mbcc=mail_bcc,
             reply_to=reply_to,
             subject=subject,
             encode=None,
             immediate=False,
             msg_type='text/html',
             charset='latin1')

    def show_prices(self, order):
        for item in order.cartitems:
            if item.show_price:
                return True
        return False

    def has_order_dimensions(self, order):
        """Checks if order has an item with dimensions.
        """
        for item in order.cartitems:
            if len(item.dimensions) > 0:
                return True
        return False

    def showStatus(self):
        registry = getUtility(IRegistry)
        shop_config = registry.forInterface(IShopConfiguration)
        return shop_config.show_status_column

    def getStatusSet(self):
        """Return the vocabulary for the currently
        selected StatusSet
        """
        status_set_name = self.shop_config.status_set
        status_set = getAdapter((self.context,), name=status_set_name)
        return status_set.vocabulary


class OrderView(BrowserView):
    """Displays a single order stored in a IOrderStorage
    """

    implements(IBrowserView, IPublishTraverse)
    template = ViewPageTemplateFile('templates/order_view.pt')

    def __init__(self, context, request):
        request.set('disable_border', True)
        registry = getUtility(IRegistry)
        self.shop_config = registry.forInterface(IShopConfiguration)
        self.order_storage_name = self.shop_config.order_storage
        super(OrderView, self).__init__(context, request)

    def __call__(self):
        new_status = self.request.form.get('form.widgets.status', '')
        try:
            new_status = int(new_status)
        except ValueError:
            new_status = None

        if new_status:
            order = self.getOrder()
            order.status = new_status
        return self.template()

    def getOrder(self, order_id=None):
        """Return the order identified by `order_id` or
        `self.order_id` (set during publishTraverse)
        """
        if not order_id:
            order_id = self.order_id
        order_storage = getUtility(IOrderStorage,
                                   name=self.order_storage_name)
        order = order_storage.getOrder(order_id)
        return order

    def has_order_dimensions(self):
        """Checks if order has an item with dimensions.
        """
        for item in self.getOrder().cartitems:
            if len(item.dimensions) > 0:
                return True
        return False

    def getStatus(self, order_id=None):
        """Return the human readable title of the status for
        this order.
        """
        order = self.getOrder()
        status = order.status

        status_set_name = self.shop_config.status_set
        status_set = getAdapter((self.context,), name=status_set_name)
        status_title = status_set.vocabulary.by_value[status].title
        return status_title

    def getStatusSet(self):
        """Return the vocabulary for the currently
        selected StatusSet
        """
        status_set_name = self.shop_config.status_set
        status_set = getAdapter((self.context,), name=status_set_name)
        return status_set.vocabulary

    def publishTraverse(self, request, id):
        """
        """
        order_view = OrderView(self.context, self.request)
        order_view.order_id = int(id)
        order_view.__name__ = str(id)
        # In Plone 3, somehow the view isn't acquisition wrapped
        # any more, therefore we fix that here
        return order_view.__of__(self.context)



from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class SupplierMailView(BrowserView):
    """Browser view for generating mail to supplier containing
    only items from the order that are being supplied by that
    supplier.
    """

    template = ViewPageTemplateFile('templates/mail/supplier_notification.pt')

    def __call__(self, show_prices=False, has_order_dimensions=True,
                 order=None, shop_config=None, supplier=None):
        self.show_prices = show_prices
        self.has_order_dimensions = has_order_dimensions
        self.order = order
        self.shop_config = shop_config
        self.supplier = supplier

        return self.template(order=order,
                             shop_config=shop_config,
                             supplier=supplier,
                             show_prices=show_prices,
                             has_order_dimensions=has_order_dimensions)

    def cartitems(self):
        """Returns the subset of items from the order that are supplied
        by the given supplier.
        """
        filtered_items = []
        supplier_name, supplier_email = self.supplier
        for item_type in self.order.cartitems:
            if item_type.supplier_email == supplier_email \
            and item_type.supplier_name == supplier_name:
                filtered_items.append(item_type)
        return filtered_items

    def show_prices(self):
           for item in self.cartitems():
               if item.show_price:
                   return True
           return False

    def getTotal(self):
        items = self.cartitems()
        prices = [i.getPrice() * i.quantity for i in items]
        total = sum(prices)
        return total



from ftw.builder import Builder
from ftw.builder import create
from ftw.shop.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestUnitField(FunctionalTestCase):

    def setUp(self):
        super(TestUnitField, self).setUp()
        self.grant('Manager')

        self.folder = create(Builder('folder'))
        self.category = create(Builder('shop category').within(self.folder))
        self.shopitem = create(Builder('shop item')
                               .titled('Raindrops')
                               .having(showPrice=True,
                                       unit=u"Buckets")
                               .within(self.category))

    @browsing
    def test_unit_is_displayed(self, browser):
        browser.login().visit(self.shopitem)

        self.assertEquals(
            'Unit',
            browser.css('.shopItemTable th').first.text)
        self.assertEquals(
            'Buckets',
            browser.css('.shopItemTable td').first.text)



from datetime import datetime
from ftw.shop.browser.cart import CartView
from ftw.shop.browser.checkout import CheckoutView
from ftw.shop.browser.checkout import CheckoutWizard
from ftw.shop.config import  ONLINE_PENDING_KEY
from ftw.shop.interfaces import IOrderStorage
from ftw.shop.tests.base import FtwShopTestCase
from ftw.shop.tests.base import MOCK_CART
from ftw.shop.tests.base import MOCK_CART_TWO_SUPPLIERS
from ftw.shop.tests.base import MOCK_CUSTOMER
from ftw.shop.tests.base import MOCK_SHIPPING
from plone import api
from zope.component import getMultiAdapter, getUtility
import unittest


class TestCheckout(FtwShopTestCase):

    def afterSetUp(self):
        super(TestCheckout, self).afterSetUp()
        self.checkout_view = CheckoutView(self.portal, self.portal.REQUEST)
        self.wizard = CheckoutWizard(self.portal, self.portal.REQUEST)

    def test_checkout_view(self):
        self.assertTrue(isinstance(self.checkout_view, CheckoutView))
        self.assertTrue(isinstance(self.portal.REQUEST['cart_view'], CartView))

    def test_wizard_steps(self):
        wizard = self.wizard
        steps = wizard.steps
        default_contact_info_step = steps[0]
        default_shipping_address_step = steps[1]
        default_payment_processor_choice_step = steps[2]
        default_order_review_step = steps[3]

        self.assertEquals(default_contact_info_step.title,
                          u'title_default_contact_info_step')
        self.assertEquals(default_shipping_address_step.title,
                        u'title_default_shipping_address_step')
        self.assertEquals(default_payment_processor_choice_step.title,
                          u'title_default_payment_processor_step')
        self.assertEquals(default_order_review_step.title,
                          u'title_default_order_review_step')

    def test_send_ordermails_one_supplier(self):
        btree_order_storage = getUtility(IOrderStorage, 'ftw.shop.BTreeOrderStorage')
        now = datetime.now()
        oid1 = btree_order_storage.createOrder(
            status=ONLINE_PENDING_KEY,
            date=now,
            customer_data=MOCK_CUSTOMER,
            shipping_data=MOCK_SHIPPING,
            cart_data=MOCK_CART,
            total='8.30')

        navroot = api.portal.get_navigation_root(self.portal)
        omanager = getMultiAdapter((navroot, self.portal.REQUEST),
                                   name=u'order_manager')

        # send order mail
        omanager.sendOrderMails(oid1)
        messages = omanager.context.MailHost.messages
        self.assertEqual(len(messages), 2)
        self.assertTrue('To: Supplier Name <supplier@example.org>' in messages[1])

    def test_send_ordermails_two_suppliers(self):
        btree_order_storage = getUtility(IOrderStorage, 'ftw.shop.BTreeOrderStorage')
        now = datetime.now()
        oid2 = btree_order_storage.createOrder(
            status=ONLINE_PENDING_KEY,
            date=now,
            customer_data=MOCK_CUSTOMER,
            shipping_data=MOCK_SHIPPING,
            cart_data=MOCK_CART_TWO_SUPPLIERS,
            total='8.30')

        navroot = api.portal.get_navigation_root(self.portal)
        omanager = getMultiAdapter((navroot, self.portal.REQUEST),
                                   name=u'order_manager')

        # send order mail
        omanager.sendOrderMails(oid2)
        messages = omanager.context.MailHost.messages
        self.assertEqual(len(messages), 3)
        self.assertTrue('To: Supplier Name <other@example.org>' in messages[1])
        self.assertTrue('To: Supplier Name <supplier@example.org>' in messages[2])


    # def test_default_contact_info_step(self):
    #     wizard = self.wizard
    #     session = self.portal.REQUEST.SESSION
    #
    #     # Test form prefill from data in session
    #     session[SESSION_ADDRESS_KEY] = MOCK_CUSTOMER
    #
    #     step = DefaultContactInfoStep(self.portal,
    #                                   self.portal.REQUEST,
    #                                   wizard)
    #     for key in MOCK_CUSTOMER.keys():
    #         self.assertEquals(step.fields[key].field.default,
    #                           MOCK_CUSTOMER[key])
    #
    #
    #     # Test form prefill from cookie
    #     cookie_data = base64.b64encode(simplejson.dumps(MOCK_CUSTOMER))
    #     self.portal.REQUEST[COOKIE_ADDRESS_KEY] = cookie_data
    #     step = DefaultContactInfoStep(self.portal,
    #                                   self.portal.REQUEST,
    #                                   wizard)
    #     for key in MOCK_CUSTOMER.keys():
    #         self.assertEquals(step.fields[key].field.default,
    #                           MOCK_CUSTOMER[key])




def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestCheckout))
    return suite



# -*- coding: utf-8 -*-
import unittest
from datetime import datetime
from decimal import Decimal

from zope.component import getUtility

from ftw.shop.config import  ONLINE_PENDING_KEY
from ftw.shop.interfaces import IOrderStorage

from ftw.shop.tests.base import FtwShopTestCase

from ftw.shop.tests.base import MOCK_CUSTOMER
from ftw.shop.tests.base import MOCK_SHIPPING
from ftw.shop.tests.base import MOCK_CART

class TestBTreeStorage(FtwShopTestCase):
    
    def afterSetUp(self):
        super(TestBTreeStorage, self).afterSetUp()

                                 
    def test_btree_order_storage(self):
        btree_order_storage = getUtility(IOrderStorage, 'ftw.shop.BTreeOrderStorage')
        now = datetime.now()
        order_id = btree_order_storage.createOrder(status=ONLINE_PENDING_KEY, 
                                                 date=now,
                                                 customer_data=MOCK_CUSTOMER,
                                                 shipping_data=MOCK_SHIPPING,
                                                 cart_data=MOCK_CART,
                                                 total='8.30')
        self.assertEquals(order_id, 1)
        
        order = btree_order_storage.getOrder(order_id)
        self.assertEquals(order.order_id, order_id)
        self.assertEquals(order.status, ONLINE_PENDING_KEY)
        self.assertEquals(order.date, now)
        self.assertEquals(order.getTotal(), Decimal('8.30'))

        order_prefix = '%03d%s' % (now.timetuple().tm_yday + 500, now.strftime("%y"))
        order_number = '%s%04d' % (order_prefix, order_id)
        self.assertEquals(order.title, order_number)
        
        self.assertEquals(order.customer_title, MOCK_CUSTOMER['title'])
        self.assertEquals(order.customer_firstname, MOCK_CUSTOMER['firstname'])
        self.assertEquals(order.customer_lastname, MOCK_CUSTOMER['lastname'])
        self.assertEquals(order.customer_street1, MOCK_CUSTOMER['street1'])
        self.assertEquals(order.customer_street2, MOCK_CUSTOMER['street2'])
        self.assertEquals(order.customer_zipcode, MOCK_CUSTOMER['zipcode'])
        self.assertEquals(order.customer_city, MOCK_CUSTOMER['city'])
        self.assertEquals(order.customer_email, MOCK_CUSTOMER['email'])
        
        cart_items = order.cartitems
        self.assertEquals(cart_items[0].sku_code, MOCK_CART['some-uid']['skucode'])
        self.assertEquals(cart_items[0].quantity, MOCK_CART['some-uid']['quantity'])
        self.assertEquals(cart_items[0].title, MOCK_CART['some-uid']['title'])
        self.assertEquals(cart_items[0].price, Decimal(MOCK_CART['some-uid']['price']))
        self.assertEquals(cart_items[0].total, Decimal(MOCK_CART['some-uid']['total']))
        self.assertEquals(cart_items[0].supplier_name, MOCK_CART['some-uid']['supplier_name'])
        self.assertEquals(cart_items[0].supplier_email, MOCK_CART['some-uid']['supplier_email'])

        self.assertEquals(cart_items[0].order_id, order_id)
        self.assertEquals(cart_items[0].order, order)
        
    def test_get_field_names(self):
        btree_order_storage = getUtility(IOrderStorage, 'ftw.shop.BTreeOrderStorage')
        expected_field_names = ['customer_city',
                                 'customer_comments',
                                 'customer_company',
                                 'customer_country',
                                 'customer_email',
                                 'customer_firstname',
                                 'customer_lastname',
                                 'customer_phone',
                                 'customer_shipping_address',
                                 'customer_street1',
                                 'customer_street2',
                                 'customer_title',
                                 'customer_zipcode',
                                 'date',
                                 'order_id',
                                 'shipping_city',
                                 'shipping_firstname',
                                 'shipping_lastname',
                                 'shipping_company',
                                 'shipping_street1',
                                 'shipping_street2',
                                 'shipping_title',
                                 'shipping_zipcode',
                                 'status',
                                 'title',
                                 'total',
                                 'vat_amount']
        field_names = btree_order_storage.getFieldNames()
        self.assertEquals(sorted(field_names), sorted(expected_field_names))



def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestBTreeStorage))
    return suite



from ftw.builder import Builder
from ftw.builder import create
from ftw.shop.testing import FTW_SHOP_FUNCTIONAL_TESTING
from ftw.shop.tests.pages import cartportlet
from ftw.shop.tests.pages import checkout
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import plone
from ftw.testbrowser.pages import z3cform
from ftw.testing.mailing import Mailing
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles
from unittest2 import TestCase
import transaction


class TestBrowserCheckout(TestCase):

    layer = FTW_SHOP_FUNCTIONAL_TESTING

    def setUp(self):
        portal = self.layer['portal']
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)

        create(Builder('cart portlet'))
        Mailing(portal).set_up()
        transaction.commit()

    def tearDown(self):
        Mailing(self.layer['portal']).tear_down()

    @browsing
    def test_contact_information_fields_required(self, browser):
        checkout.visit_checkout_with_one_item_in_cart()
        checkout.next()
        checkout.assert_step(checkout.CONTACT_INFORMATION)
        form = browser.css('form.kssattr-formname-checkout-wizard').first

        self.assertEquals(
            {u'Title': ['Required input is missing.'],
             u'First Name': ['Required input is missing.'],
             u'Last Name': ['Required input is missing.'],
             u'Email': ['Required input is missing.'],
             u'Street/No.': ['Required input is missing.'],
             u'Phone number': ['Required input is missing.'],
             u'Zip Code': ['Required input is missing.'],
             u'City': ['Required input is missing.'],
             u'Country': ['Required input is missing.']},
            z3cform.erroneous_fields(form))

    @browsing
    def test_valid_email_address_required(self, browser):
        checkout.visit_checkout_with_one_item_in_cart()
        browser.fill({'Email': 'invalid.email.ch'})
        checkout.next()
        checkout.assert_step(checkout.CONTACT_INFORMATION)

        form = browser.css('form.kssattr-formname-checkout-wizard').first
        self.assertEquals(
            ['This email address is invalid.'],
            z3cform.erroneous_fields(form)[u'Email'])

    @browsing
    def test_contact_information_defaults_for_logged_in_user(self, browser):
        hugo = create(Builder('user').named('H\xc3\xbcgo', 'B\xc3\xb6ss'))
        browser.login(hugo)

        checkout.visit_checkout_with_one_item_in_cart()
        checkout.next()
        checkout.assert_step(checkout.CONTACT_INFORMATION)

        self.assertEquals(
            hugo.getProperty('fullname').split(' ')[1].decode('utf-8'),
            browser.css(
                '[name="contact_information.widgets.lastname"]').first.value)
        self.assertEquals(
            hugo.getProperty('fullname').split(' ')[0].decode('utf-8'),
            browser.css(
                '[name="contact_information.widgets.firstname"]').first.value)

    @browsing
    def test_contact_information_defaults_for_user_without_fullname(self,
                                                                    browser):
        browser.login()
        checkout.visit_checkout_with_one_item_in_cart()
        checkout.next()
        checkout.assert_step(checkout.CONTACT_INFORMATION)

        self.assertEquals(
            '',
            browser.css(
                '[name="contact_information.widgets.lastname"]').first.value)
        self.assertEquals(
            '',
            browser.css(
                '[name="contact_information.widgets.firstname"]').first.value)


    @browsing
    def test_submitting_contact_information_leads_to_shipping_address(self, browser):
        checkout.visit_checkout_with_one_item_in_cart()
        checkout.submit_valid_contact_info()
        checkout.assert_step(checkout.SHIPPING_ADDRESS)

    @browsing
    def test_shipping_address_is_prefilled(self, browser):
        checkout.visit_checkout_with_one_item_in_cart()
        checkout.fill_contact_info()
        browser.fill({'First Name': 'John',
                      'Last Name': 'Doe'})
        checkout.next()
        checkout.assert_step(checkout.SHIPPING_ADDRESS)
        self.assertEquals('John', browser.find('First Name').value)
        self.assertEquals('Doe', browser.find('Last Name').value)

    @browsing
    def test_back_on_shipping_address(self, browser):
        checkout.goto(checkout.SHIPPING_ADDRESS)
        checkout.back()
        checkout.assert_step(checkout.CONTACT_INFORMATION)

    @browsing
    def test_payment_processor_is_required(self, browser):
        checkout.goto(checkout.PAYMENT_PROCESSOR)
        checkout.next()
        checkout.assert_step(checkout.PAYMENT_PROCESSOR)
        form = browser.css('form.kssattr-formname-checkout-wizard').first

        # XXX: "Gegen Rechnung" should be translated to english
        self.assertEquals({u'Payment Processor': ['Required input is missing.']},
                          z3cform.erroneous_fields(form))

    @browsing
    def test_payment_processor_selection(self, browser):
        checkout.goto(checkout.PAYMENT_PROCESSOR)
        checkout.submit_valid_payment_processor()
        checkout.assert_step(checkout.ORDER_REVIEW)

    @browsing
    def test_back_on_payment_processor(self, browser):
        checkout.goto(checkout.PAYMENT_PROCESSOR)
        checkout.select_valid_payment_processor()
        checkout.back()
        checkout.assert_step(checkout.SHIPPING_ADDRESS)

    @browsing
    def test_order_review_shows_contact_information(self, browser):
        checkout.goto(checkout.ORDER_REVIEW)
        self.assertEquals(['Sir',
                           'Hugo Boss',
                           '',
                           'Example Street 15',
                           '',
                           '3000 Bern',
                           'Switzerland',
                           '',
                           'Email: hugo@boss.com',
                           'Phone: 001 0101 0101 01',
                           ''],
                          checkout.review_contact_information())

    @browsing
    def test_order_review_shows_shipping_address(self, browser):
        checkout.goto(checkout.ORDER_REVIEW)
        self.assertEquals(['Sir',
                           'Hugo Boss',
                           '',
                           'Example Street 15',
                           '',
                           '3000 Bern',
                           ''],
                          checkout.review_shipping_address())

    @browsing
    def test_order_review_shows_cart_items(self, browser):
        item = create(Builder('shop item')
                      .titled('Socks')
                      .having(description='A good pair of socks.',
                              price='10'))
        checkout.visit_checkout_with_one_item_in_cart(item=item)
        checkout.goto(checkout.ORDER_REVIEW)
        self.assertEquals([{'Product': 'Socks',
                            'Description': 'A good pair of socks.',
                            'Quantity': '1',
                            'Price per item': '10.00',
                            'Total': '10.00'}],
                          browser.css('table.cartListing').first.dicts(foot=False))

    @browsing
    def test_order_review_shows_correct_total(self, browser):
        pants = create(Builder('shop item').titled('Fancy Pants').having(price='35'))
        checkout.add_item_to_cart(pants, amount=2)
        socks = create(Builder('shop item').titled('Socks').having(price='12'))
        checkout.add_item_to_cart(socks, amount=3)

        cartportlet.order_link().click()
        checkout.goto(checkout.ORDER_REVIEW)

        # pants:  2 * 35 = 70
        # socks:  3 * 12 = 36
        # total          = 106

        table = browser.css('table.cartListing').first
        self.assertEquals([['VAT', '', '', '', '0.00'],
                           ['Total', '', '', '', '106.00']],
                          table.lists(head=False, body=False, foot=True))

    @browsing
    def test_back_on_order_review_leads_to_payment_processor(self, browser):
        checkout.goto(checkout.ORDER_REVIEW)
        checkout.back()
        checkout.assert_step(checkout.PAYMENT_PROCESSOR)

    @browsing
    def test_thankyou_page_when_finishing_checkout(self, browser):
        checkout.goto(checkout.ORDER_REVIEW).finish()
        self.assertEquals('thankyou', plone.view())



from decimal import Decimal
from ftw.shop.config import SESSION_ADDRESS_KEY
from ftw.shop.config import SESSION_SHIPPING_KEY
from ftw.shop.interfaces import IShoppingCart
from ftw.shop.tests.base import FtwShopTestCase
from plone import api
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
import simplejson
import unittest


class TestCart(FtwShopTestCase):

    def afterSetUp(self):
        super(TestCart, self).afterSetUp()

    def test_add_to_cart(self):
        # Add an item with no variations to cart
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart.addtocart("12345", quantity=2, dimension=[Decimal(1), Decimal(2)])
        cart_items = cart.cart_items()
        self.assertEquals(len(cart_items), 1)
        item_key = self.movie.UID() + '==1-2'
        self.assertEquals(cart_items[item_key]['price'], '7.15')
        self.assertEquals(cart_items[item_key]['price_per_item'], '14.30')
        self.assertEquals(cart_items[item_key]['total'], '28.60')
        self.assertEquals(cart_items[item_key]['skucode'], '12345')
        self.assertEquals(cart_items[item_key]['quantity'], 2)
        self.assertEquals(cart_items[item_key]['title'], 'A Movie')
        self.assertEquals(cart_items[item_key]['description'], 'A Shop Item with no variations')
        self.assertEquals(cart.cart_total(), '28.60')

        # Add an item type that's already in the cart
        cart.addtocart("12345", quantity=1, dimension=[Decimal(1), Decimal(2)])
        self.assertEquals(cart.cart_items()[item_key]['quantity'], 3)
        self.assertEquals(cart.cart_total(), '42.90')


        # Add an item with one variation to cart
        cart = getMultiAdapter((self.book, self.portal.REQUEST), name='cart_view')
        cart.addtocart(var1choice='Paperback', quantity=3)
        cart_items = cart.cart_items()
        self.assertEquals(len(cart_items), 2)
        item_key = "%s=var-Paperback=" % self.book.UID()

        self.assertEquals(cart_items[item_key]['price'], '2.00')
        self.assertEquals(cart_items[item_key]['price_per_item'], '2.00')
        self.assertEquals(cart_items[item_key]['total'], '6.00')
        self.assertEquals(cart_items[item_key]['skucode'], 'b22')
        self.assertEquals(cart_items[item_key]['quantity'], 3)
        self.assertEquals(cart_items[item_key]['title'], 'Professional Plone Development - None')
        self.assertEquals(cart_items[item_key]['description'], 'A Shop Item with one variation')

        self.assertEquals(cart.cart_total(), '48.90')

        # Add an item type with variations that's already in the cart
        cart.addtocart(var1choice='Paperback', quantity=1)
        self.assertEquals(cart.cart_items()[item_key]['quantity'], 4)
        self.assertEquals(cart.cart_total(), '50.90')


        # Add an item with two variations to cart
        cart = getMultiAdapter((self.tshirt, self.portal.REQUEST), name='cart_view')
        cart.addtocart(var1choice='Green', var2choice='M', quantity=4)
        cart_items = cart.cart_items()
        self.assertEquals(len(cart_items), 3)
        item_key = "%s=var-Green-M=" % self.tshirt.UID()
        self.assertEquals(cart_items[item_key]['price'], '5.00')
        self.assertEquals(cart_items[item_key]['price_per_item'], '5.00')
        self.assertEquals(cart_items[item_key]['total'], '20.00')
        self.assertEquals(cart_items[item_key]['skucode'], '55')
        self.assertEquals(cart_items[item_key]['quantity'], 4)
        self.assertEquals(cart_items[item_key]['title'], 'A T-Shirt - None')
        self.assertEquals(cart_items[item_key]['description'], 'A Shop Item with two variations')

        self.assertEquals(cart.cart_total(), '70.90')
        cart.cart_delete()

    def test_add_with_different_and_same_dimensions(self):
        # Add an item with dimensions
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart.addtocart("12345", quantity=1, dimension=[Decimal(1), Decimal(2)])
        cart_items = cart.cart_items()
        item_key = self.movie.UID() + '==1-2'

        self.assertEquals(1, len(cart_items))

        # adding with same dimensions does not add a second item in the cart
        cart.addtocart("12345", quantity=2, dimension=[Decimal(1), Decimal(2)])
        cart_items = cart.cart_items()
        self.assertEquals(1, len(cart_items))
        self.assertEquals(3, cart_items[item_key]['quantity'])

        # adding with different dimensions adds a second item
        cart.addtocart("12345", quantity=1, dimension=[Decimal(3), Decimal(3)])
        cart_items = cart.cart_items()
        self.assertEquals(2, len(cart_items))

        cart.cart_delete()

    def test_add_to_cart_ajax(self):
        # Add an item with no variations to cart
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart_response = cart.addtocart_ajax("12345", quantity=2, dimensions='1|2')
        cart_response_dict = simplejson.loads(cart_response)
        portlet_html = cart_response_dict['portlet_html']
        status_message = cart_response_dict['status_message']
        self.assertEquals(status_message, "<dt>Information</dt>\n<dd>Added item to cart.</dd>")
        self.assertTrue('<dl class="portlet portletCartPortlet">' in portlet_html)
        self.assertTrue('28.60' in portlet_html)
        self.assertTrue('(2)' in portlet_html)
        self.assertTrue('<div class="cartTotal">Total: CHF 28.60</div>' in portlet_html)
        self.assertEquals(cart.cart_total(), '28.60')
        cart.cart_delete()

    def test_remove_item(self):
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart_item_count = len(cart.cart_items())
        cart.addtocart(skuCode='12345', dimension=[Decimal(1), Decimal(2)])
        movie_key = self.movie.UID()+'==1-2'
        self.assertEquals(len(cart.cart_items()), cart_item_count + 1)
        self.assertTrue(movie_key in cart.cart_items())
        cart.cart._remove_item(movie_key)
        self.assertEquals(len(cart.cart_items()), cart_item_count)
        self.assertTrue(movie_key not in cart.cart_items())

    def test_update_item(self):
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart.addtocart(skuCode='12345', quantity=1, dimension=[Decimal(1), Decimal(2)])
        movie_key = self.movie.UID()+'==1-2'
        cart.update_item(movie_key, 2, [2, 2])
        movie_key = self.movie.UID() + '==2-2'
        self.assertEquals(cart.cart_items()[movie_key]['quantity'], 2)
        self.assertEquals(cart.cart_items()[movie_key]['price_per_item'], '28.60')
        self.assertEquals(cart.cart_items()[movie_key]['total'], '57.20')

    def test_cart_update(self):
        ptool = getToolByName(self.portal, 'plone_utils')
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart.addtocart(skuCode='12345', quantity=1, dimension=[Decimal(1), Decimal(2)])
        cart = getMultiAdapter((self.book, self.portal.REQUEST), name='cart_view')
        cart.addtocart(skuCode='b11', var1choice='Hardcover', quantity=2)

        # Try to update cart with no values in request (invalid)
        cart.cart_update()
        last_msg = ptool.showPortalMessages()[-1].message
        self.assertEquals(last_msg, u'Invalid Values specified. Cart not updated.')
        self.assertEquals(len(cart.cart_items()), 2)

        # Modify quantities
        self.portal.REQUEST['quantity_%s==1-2' % self.movie.UID()] = 2
        self.portal.REQUEST['dimension_%s==1-2' % self.movie.UID()] = [1, 2]
        self.portal.REQUEST['quantity_%s=var-Hardcover=' % self.book.UID()] = 0
        cart.cart_update()
        self.assertEquals(cart.cart_items()[self.movie.UID()+'==1-2']['quantity'], 2)
        self.assertTrue('b11' not in cart.cart_items())
        last_msg = ptool.showPortalMessages()[-1].message
        self.assertEquals(last_msg, u'Cart updated.')

    def test_cart_item_order(self):
        # Previously the order of the items was random. This test failed about
        # half the times it was run. The OrderedDict change fixed it.
        ptool = getToolByName(self.portal, 'plone_utils')
        cart = getMultiAdapter((self.book, self.portal.REQUEST), name='cart_view')
        cart.addtocart(skuCode='b11', var1choice='Hardcover', quantity=2)
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart.addtocart(skuCode='12345', quantity=1, dimension=[Decimal(1), Decimal(2)])
        cart.cart_update()
        self.assertEquals(
            ['b11', '12345'],
            [item['skucode'] for item in cart.cart_items().values()])

    def test_changing_dimensions_updates_key(self):
        # Add an item with dimensions
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart.addtocart("12345", quantity=1, dimension=[Decimal(1), Decimal(2)])
        cart_items = cart.cart_items()

        self.assertEquals(1, len(cart_items))

        # updating the dimensions will change the key
        self.portal.REQUEST['dimension_%s==1-2' % self.movie.UID()] = [3, 3]
        self.portal.REQUEST['quantity_%s==1-2' % self.movie.UID()] = 1
        cart.cart_update()

        self.assertEquals([self.movie.UID()+'==3-3'], cart.cart_items().keys())

        # add a second item and change it to the same dimensions
        # as the first item
        cart.addtocart("12345", quantity=2, dimension=[Decimal(1), Decimal(2)])

        self.assertEquals(2, len(cart.cart_items()))

        self.portal.REQUEST['dimension_%s==1-2' % self.movie.UID()] = [3, 3]
        self.portal.REQUEST['quantity_%s==1-2' % self.movie.UID()] = 2
        self.portal.REQUEST['dimension_%s==3-3' % self.movie.UID()] = [3, 3]
        self.portal.REQUEST['quantity_%s==3-3' % self.movie.UID()] = 1
        cart.cart_update()

        ptool = getToolByName(self.portal, 'plone_utils')
        last_msg = ptool.showPortalMessages()[-1].message
        self.assertEquals(last_msg, u'Invalid Values specified. Cart not updated.')

        cart_items = cart.cart_items()
        self.assertEquals(2, len(cart_items))
        self.assertEquals(1, cart_items[self.movie.UID()+'==3-3']['quantity'])

    def test_cart_remove(self):
        ptool = getToolByName(self.portal, 'plone_utils')
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart.addtocart('12345', dimension=[Decimal(1), Decimal(2)])

        # Remove the item we just added
        item_key = self.movie.UID() + '==1-2'
        # XXX - this is not an skuCode but an item key - rename!
        self.portal.REQUEST['skuCode'] = item_key
        cart.cart_remove()
        self.assertEquals(len(cart.cart_items()), 0)
        last_msg = ptool.showPortalMessages()[-1].message
        self.assertEquals(last_msg, u"Cart updated.")

    def test_cart_delete(self):
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')
        cart.addtocart('12345', dimension=[Decimal(1), Decimal(2)])
        self.assertEquals(len(cart.cart_items()), 1)

        cart.cart_delete()
        self.assertEquals(len(cart.cart_items()), 0)

    def test_checkout(self):
        ptool = getToolByName(self.portal, 'plone_utils')
        pp_choice = {'payment_processor': 'ftw.shop.InvoicePaymentProcessor'}
        customer_info = {'title': 'Mr.',
                         'firstname': "Hugo",
                         'lastname': "Boss",
                         'email': 'hugo.boss@example.org'}
        shipping_info = {'title': 'Mr.',
                         'firstname': "Hugo",
                         'lastname': "Boss"}
        cart = getMultiAdapter((self.movie, self.portal.REQUEST), name='cart_view')

        # Test checkout with empty cart
        cart.checkout()
        last_msg = ptool.showPortalMessages()[-1].message
        self.assertEquals(last_msg, u"Can't proceed with empty cart.")

        # Test checkout with item in cart but missing order confirmation
        cart.addtocart('12345', dimension=[Decimal(1), Decimal(2)])
        self.portal.REQUEST.SESSION[SESSION_ADDRESS_KEY] = customer_info
        cart.checkout()

        # Test checkout with item in cart but missing payment processor
        self.portal.REQUEST.SESSION[SESSION_ADDRESS_KEY] = customer_info
        self.portal.REQUEST.SESSION['order_confirmation'] = True
        cart.checkout()

        # Test checkout with item and all necessary information
        navroot = api.portal.get_navigation_root(self.movie)
        omanager = getMultiAdapter((navroot, self.portal.REQUEST),
                                   name=u'order_manager')

        self.portal.REQUEST.SESSION[SESSION_ADDRESS_KEY] = customer_info
        self.portal.REQUEST.SESSION[SESSION_SHIPPING_KEY] = shipping_info
        self.portal.REQUEST.SESSION['order_confirmation'] = True
        self.portal.REQUEST.SESSION['payment_processor_choice'] = pp_choice
        cart.checkout()

        order = omanager.getOrders()[0]
        self.assertEquals(order.total, Decimal('14.30'))
        self.assertEquals(order.status, 3)
        self.assertEquals(order.customer_firstname, "Hugo")
        self.assertEquals(order.customer_lastname, "Boss")
        self.assertEquals(order.customer_email, "hugo.boss@example.org")

    def test_calc_item_total(self):
        cart = getMultiAdapter((self.portal, self.portal.REQUEST), IShoppingCart)
        self.assertEqual(
            7.5,
            cart.calc_item_total(7.5, 1, [1], 1),
            "Price should be respected in the calculation")
        self.assertEqual(
            15,
            cart.calc_item_total(7.5, 2, [1], 1),
            "Quantity should be respected in the calculation")
        self.assertEqual(
            15,
            cart.calc_item_total(7.5, 1, [2], 1),
            "Dimension should be respected in the calculation")
        self.assertEqual(
            60,
            cart.calc_item_total(7.5, 1, [2, 2, 2], 1),
            "Multiple dimensions should be respected in the calculation")
        self.assertEqual(
            120,
            cart.calc_item_total(7.5, 2, [2, 2, 2], 1),
            "Quantity and dimensions can be used simultaneously")
        self.assertEqual(
            4.4,
            float(cart.calc_item_total(11, 2, [0.2, 1, 1], 1)),
            "Decimal dimensions should be respected in the calculation.")
        self.assertEqual(
            2.2,
            # 11.-/m2 and 10mm * 1000mm dimensions
            float(cart.calc_item_total(11, 2, [100, 1000], 1000000)),
            "The price modifier is used to convert price unit to dimension unit.")


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestCart))
    return suite



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.z3cform.layout import wrap_form
from z3c.form import field, group
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper

from ftw.shop.interfaces import IShopConfiguration
from ftw.shop import shopMessageFactory as _


class MailGroup(group.Group):
    label = _('label_mail_group', default=u'Mail')
    fields = field.Fields(IShopConfiguration).select(
        'shop_email',
        'mail_bcc',
        'mail_subject',
        'always_notify_shop_owner')


class CheckoutGroup(group.Group):
    label = _('label_checkout_group', default=u'Checkout')
    fields = field.Fields(IShopConfiguration).select(
        'payment_processor_step_group',
        'enabled_payment_processors',
        'contact_info_step_group',
        'shipping_address_step_group',
        'order_review_step_group',
        'phone_number')


class VATGroup(group.Group):
    label = _('label_vat_group', default=u'VAT')
    fields = field.Fields(IShopConfiguration).select(
        'vat_enabled',
        'vat_number',
        'vat_rates')


class ShopConfigurationForm(RegistryEditForm, group.GroupForm):
    """Configuration form for the ftw.shop configlet
    """
    schema = IShopConfiguration
    fields = field.Fields(IShopConfiguration).omit('shop_email',
                                                   'mail_bcc',
                                                   'mail_subject',
                                                   'always_notify_shop_owner',
                                                   'payment_processor_step_group',
                                                   'enabled_payment_processors',
                                                   'contact_info_step_group',
                                                   'shipping_address_step_group',
                                                   'order_review_step_group',
                                                   'phone_number',
                                                   'vat_enabled',
                                                   'vat_number',
                                                   'vat_rates')
    groups = (MailGroup, CheckoutGroup, VATGroup)
    label = _(u'label_shop_configuration', default=u"Shop configuration")

    def updateFields(self):
        super(ShopConfigurationForm, self).updateFields()

    def updateWidgets(self):
        super(ShopConfigurationForm, self).updateWidgets()

# BBB: For Plone3 compatibility, we need to override the template
# on the form wrapper, not the form itself, and use zope2's
# ViewPageTemplateFile, not zope3's
# See http://plone.org/products/dexterity/documentation/error/rendering-a-form-fails-with-attributeerror-str-object-has-no-attribute-other

ShopConfigurationView = wrap_form(ShopConfigurationForm, ControlPanelFormWrapper)
ShopConfigurationView.template = ViewPageTemplateFile('templates/controlpanel.pt')



from ftw.builder import Builder
from ftw.builder import create
from ftw.shop.interfaces import IShopConfiguration
from ftw.shop.interfaces import IShopRoot
from ftw.shop.testing import FTW_SHOP_FUNCTIONAL_TESTING
from ftw.shop.tests.helpers import get_mail_header
from ftw.shop.tests.pages import checkout
from ftw.testbrowser import browsing
from ftw.testing.mailing import Mailing
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.registry.interfaces import IRegistry
from unittest2 import TestCase
from zope.component import getUtility
from zope.interface import alsoProvides
import email
import email.header


class TestCheckoutMailToShopOwner(TestCase):

    layer = FTW_SHOP_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        registry = getUtility(IRegistry)
        settings = registry.forInterface(IShopConfiguration)
        settings.always_notify_shop_owner = True

        alsoProvides(self.portal, IShopRoot)
        create(Builder('cart portlet'))
        Mailing(self.portal).set_up()

    def checkout_and_get_mail(self, *items):
        mailing = Mailing(self.portal)
        for item in items:
            checkout.visit_checkout_with_one_item_in_cart(item)
        checkout.goto(checkout.ORDER_REVIEW).finish()

        mails_by_recipient = mailing.get_messages_by_recipient()
        owner = 'Shop Owner <webshop@example.org>'
        self.assertIn(owner, mails_by_recipient,
                      'Expected a mail to the shop owner to be sent')
        return email.message_from_string(mails_by_recipient[owner][0])

    def open_mail_in_browser(self, mail, browser):
        browser.open_html(mail.get_payload(decode=True))

    @browsing
    def test_order_details(self, browser):
        self.open_mail_in_browser(self.checkout_and_get_mail(), browser)
        self.assertRegexpMatches(browser.css('#order-details').first.text,
                                 '\n'.join((r'^Order number: \d*',
                                            r'Date: [^\n]*',
                                            r'Order Status: 3$')))

    @browsing
    def test_shop_owner_is_notified(self, browser):
        mail = self.checkout_and_get_mail()
        self.assertEquals(['Shop Owner <webshop@example.org>'],
                          get_mail_header(mail, 'To'))

        self.assertRegexpMatches(get_mail_header(mail, 'Subject')[0],
                                 r'^\[Webshop\] Order \d* by Hugo Boss$')

    @browsing
    def test_shop_owner_replies_to_customer(self, browser):
        mail = self.checkout_and_get_mail()
        self.assertEquals(['Hugo Boss <hugo@boss.com>'],
                          get_mail_header(mail, 'Reply-To'))

    @browsing
    def test_personal_information(self, browser):
        self.open_mail_in_browser(self.checkout_and_get_mail(), browser)

        self.assertMultiLineEqual(
            '\n'.join(('Sir',
                       'Hugo Boss',
                       'Example Street 15',
                       '3000 Bern',
                       '',
                       'Email: hugo@boss.com',
                       'Phone: 001 0101 0101 01')),

            browser.css('#personal-information').first.text)

    @browsing
    def test_shipping_address(self, browser):
        self.open_mail_in_browser(self.checkout_and_get_mail(), browser)

        self.assertMultiLineEqual(
            '\n'.join(('Sir',
                       'Hugo Boss',
                       'Example Street 15',
                       '3000 Bern',
                       'Switzerland')),

            browser.css('#shipping-address').first.text)

    @browsing
    def test_product_is_listed_in_order_table(self, browser):
        item = create(Builder('shop item')
                      .titled('A pair of socks')
                      .having(skuCode='123.1',
                              price='12.90',
                              showPrice=True))

        self.open_mail_in_browser(self.checkout_and_get_mail(item), browser)

        self.assertEquals([{'Article number': '123.1',
                            'Product': 'A pair of socks',
                            'Price per item': '12.90',
                            'Quantity': '1',
                            'Total': '12.90'}],
                          browser.css('table').first.dicts(foot=False))

    @browsing
    def test_total_price_is_correct_in_order_table(self, browser):
        items = [create(Builder('shop item').having(price='10.00')),
                 create(Builder('shop item').having(price='5.00'))]

        self.open_mail_in_browser(self.checkout_and_get_mail(*items), browser)

        self.assertEquals([['VAT', '', '', '', '0.00'],
                           ['Total (incl.VAT)', '', '', '', '15.00']],
                          browser.css('table').first.lists(head=False, body=False))



from ftw.shop.tests.base import FtwShopTestCase


class TestShopIndexer(FtwShopTestCase):

    def test_sku_code_is_searchable(self):
        # default skuCode on ShopItem
        self.assertIn(self.movie.skuCode, self.movie.SearchableText())

        # skuCode in variation
        self.assertIn('b11', self.book.SearchableText())
        self.assertIn('b22', self.book.SearchableText())



import unittest
from decimal import Decimal
from ftw.shop.tests.base import FtwShopTestCase

class TestVariations(FtwShopTestCase):
    """Here, we test the different variation methods with three items
    that are set up in FtwShopTestCase.afterSetUp():
    movie (no variations), book (one variation) and tshirt (two variations)
    """
    
    def afterSetUp(self):
        super(TestVariations, self).afterSetUp()

    def test_has_variations(self):
        self.assertFalse(self.movie_vc.hasVariations())
        self.assertTrue(self.book_vc.hasVariations())
        self.assertTrue(self.tshirt_vc.hasVariations())

    def test_get_item_uid(self):
        self.assertEquals(self.movie_vc.getItemUID(), self.movie.UID())
        self.assertEquals(self.book_vc.getItemUID(), self.book.UID())
        self.assertEquals(self.tshirt_vc.getItemUID(), self.tshirt.UID())

    def test_get_var1_values(self):
        movie_var1_values = self.movie_vc.getVariation1Values()
        book_var1_values = self.book_vc.getVariation1Values()
        tshirt_var1_values = self.tshirt_vc.getVariation1Values()
        self.assertEquals(movie_var1_values, [])
        self.assertEquals(book_var1_values, ('Hardcover', 'Paperback'))
        self.assertEquals(tshirt_var1_values, ('Red', 'Green', 'Blue'))

    def test_get_var2_values(self):
        movie_var2_values = self.movie_vc.getVariation2Values()
        book_var2_values = self.book_vc.getVariation2Values()
        tshirt_var2_values = self.tshirt_vc.getVariation2Values()
        self.assertEquals(movie_var2_values, [])
        self.assertEquals(book_var2_values, [])
        self.assertEquals(tshirt_var2_values, ('S', 'M', 'L'))

    def test_get_variation_attributes(self):
        movie_var_attrs = self.movie_vc.getVariationAttributes()
        book_var_attrs = self.book_vc.getVariationAttributes()
        tshirt_var_attrs = self.tshirt_vc.getVariationAttributes()
        self.assertEquals(movie_var_attrs, [])
        self.assertEquals(book_var_attrs, ['Cover'])
        self.assertEquals(tshirt_var_attrs, ['Color', 'Size'])

    def test_get_variation_data(self):
        price = self.movie_vc.getVariationData(None, None, 'price')
        skuCode = self.movie_vc.getVariationData(None, None, 'skuCode')
        active = self.movie_vc.getVariationData(None, None, 'active')
        hasUniqueSKU = self.movie_vc.getVariationData(None, None, 'hasUniqueSKU')
        not_there = self.movie_vc.getVariationData(None, None, 'nonexisting')

        self.assertEquals(price, Decimal('7.15'))
        self.assertEquals(skuCode, '12345')
        self.assertEquals(active, True)
        self.assertEquals(hasUniqueSKU, False)
        self.assertEquals(not_there, None)

        price = self.book_vc.getVariationData('Paperback', None, 'price')
        skuCode = self.book_vc.getVariationData('Paperback', None, 'skuCode')
        active = self.book_vc.getVariationData('Paperback', None, 'active')
        self.assertEquals(price, Decimal('2.00'))
        self.assertEquals(skuCode, 'b22')
        self.assertTrue(active)

        price = self.tshirt_vc.getVariationData('Blue', 'S', 'price')
        skuCode = self.tshirt_vc.getVariationData('Blue', 'S', 'skuCode')
        active = self.tshirt_vc.getVariationData('Blue', 'S', 'active')
        self.assertEquals(price, Decimal('7.00'))
        self.assertEquals(skuCode, '77')
        self.assertTrue(active)

    def test_get_pretty_name(self):
        self.assertEquals(self.book_vc.getPrettyName('var-0'), 'Hardcover')
        self.assertEquals(self.tshirt_vc.getPrettyName('var-2-0'), 'Blue-S')
        self.assertEquals(self.tshirt_vc.getPrettyName('doesnt-exist'), None)

    def test_get_variation_dict(self):
        expected_var_dict = {
        'var-Hardcover': {'active': True, 'price': Decimal('1.00'), 'hasUniqueSKU': True, 'description': 'A hard and durable cover', 'skuCode': 'b11'},
        'var-Paperback': {'active': True, 'price': Decimal('2.00'), 'hasUniqueSKU': True, 'description': 'A less durable but cheaper cover', 'skuCode': 'b22'},
        }
        self.assertEquals(self.book_vc.getVariationDict(),
                          expected_var_dict)

        expected_var_dict = {
        'var-Red-S': {'active': True, 'price': Decimal('1.00'), 'skuCode': '11', 'hasUniqueSKU': True, 'description': ''},
        'var-Red-M': {'active': True, 'price': Decimal('2.00'), 'skuCode': '22', 'hasUniqueSKU': True, 'description': ''}, 
        'var-Red-L': {'active': True, 'price': Decimal('3.00'), 'skuCode': '33', 'hasUniqueSKU': True, 'description': ''},
        'var-Green-S': {'active': True, 'price': Decimal('4.00'), 'skuCode': '44', 'hasUniqueSKU': True, 'description': ''},
        'var-Green-M': {'active': True, 'price': Decimal('5.00'), 'skuCode': '55', 'hasUniqueSKU': True, 'description': ''}, 
        'var-Green-L': {'active': True, 'price': Decimal('6.00'), 'skuCode': '66', 'hasUniqueSKU': True, 'description': ''}, 
        'var-Blue-S': {'active': True, 'price': Decimal('7.00'), 'skuCode': '77', 'hasUniqueSKU': True, 'description': ''}, 
        'var-Blue-M': {'active': True, 'price': Decimal('8.00'), 'skuCode': '88', 'hasUniqueSKU': True, 'description': ''}, 
        'var-Blue-L': {'active': True, 'price': Decimal('9.00'), 'skuCode': '99', 'hasUniqueSKU': True, 'description': ''},
        }

        self.assertEquals(self.tshirt_vc.getVariationDict(), expected_var_dict)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestVariations))
    return suite



"""Common configuration constants
"""
from Products.Archetypes.public import DisplayList
from decimal import Decimal

PROJECTNAME = 'ftw.shop'

ADD_PERMISSIONS = {
    'ShopCategory': 'ftw.shop: Add Shop Category',
    'ShopItem': 'ftw.shop: Add Shop Item',
    'Supplier': 'ftw.shop: Add Supplier',
}

CURRENCIES = DisplayList((
    ('CHF', 'CHF'), # CHF
#      ('USD', 'US$'), # U.S. Dollars
#      ('EUR', 'Eur'), # Euros
))

SESSION_ORDERS_KEY = 'ftw.shop.orders'
SESSION_ADDRESS_KEY = 'ftw.shop.customer_data'
SESSION_SHIPPING_KEY = 'ftw.shop.shipping_address'
SESSION_REVIEW_KEY = 'ftw.shop.review_data'
SESSION_ERRORS_KEY = 'ftw.shop.errors'

COOKIE_ADDRESS_KEY = 'ftw.shop.customer_cookie'

ONLINE_PENDING_KEY = 1
ONLINE_CONFIRMED_KEY = 2
ONACCOUNT_KEY = 3

CART_KEY = 'shop_cart_items'

CATEGORY_RELATIONSHIP = 'shop_category'

DEFAULT_VAT_RATES = [Decimal('0.0'), Decimal('8.0'), Decimal('2.5')]



# -*- coding: utf-8 -*-
"""
This module contains the tool of ftw.shop
"""
import os
from setuptools import setup, find_packages

version = '3.0.2'

tests_require = [
    'Products.PloneTestCase',
    'ftw.builder',
    'ftw.testbrowser',
    'ftw.testing',
    'plone.app.testing',
    'pyquery',
    'zope.testing',
    ]

setup(name='ftw.shop',
      version=version,
      description="A web shop solution for Plone",
      long_description=open("README.rst").read() + "\n" + \
          open(os.path.join("docs", "HISTORY.txt")).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='ftw shop plone',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.shop',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw', ],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'Plone',
        'collective.js.jqueryui',
        'collective.z3cform.wizard',
        'ftw.upgrade >= 1.14.5',
        'plone.api',
        'plone.app.registry',
        'plone.app.z3cform',
        'setuptools',
        'simplejson',
        ],

      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite = 'ftw.shop.tests.test_docs.test_suite',

      entry_points="""
      # -*- entry_points -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from ftw.builder import Builder
from ftw.builder import create
from ftw.shop.browser.cart import validate_dimensions
from ftw.shop.content.shopitem import selectable_dimensions
from ftw.shop.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestDimensions(FunctionalTestCase):

    def setUp(self):
        super(TestDimensions, self).setUp()
        self.grant('Manager')

        self.folder = create(Builder('folder'))
        self.category = create(Builder('shop category').within(self.folder))
        self.shopitem = create(Builder('shop item')
                               .titled('Raindrops')
                               .having(price='2.00',
                                       showPrice=True,
                                       selectable_dimensions='length_width_mm_mm2')
                               .within(self.category))

    @browsing
    def test_dimension_input(self, browser):
        browser.login().visit(self.shopitem)

        self.assertEquals(
            2,
            len(browser.css('.dimensions-selection input[name="dimension:float"]')))

    @browsing
    def test_dimension_add_and_edit(self, browser):
        browser.login().visit(self.shopitem)

        dim = browser.css('.dimensions-selection input[name="dimension:float"]')
        dim[0].value = '2.5'
        dim[1].value = '3'
        browser.fill({'quantity:int': '1'}).submit()

        browser.open(self.portal, view='cart_edit')

        dim = browser.css('.dimensions-selection input')
        self.assertEquals('2.5', dim[0].value)
        self.assertEquals('3', dim[1].value)
        # check if price is correctly multiplied
        self.assertEquals('15.00', browser.css('form > div b span').first.text)

        dim[1].value = '4'

        browser.fill({'cart_update:method': 'submitted'}).submit()

        dim = browser.css('.dimensions-selection input')
        self.assertEquals('4', dim[1].value)
        self.assertEquals('20.00', browser.css('form > div b span').first.text)

    def test_dimensions_validation(self):
        self.assertTrue(validate_dimensions([], []))
        self.assertTrue(validate_dimensions([1], [u'Length (mm)']))
        self.assertTrue(validate_dimensions(
            [1, 2, 3],
            [u"Length (mm)", u"Width (mm)", u"Thickness (mm)"]))
        self.assertTrue(validate_dimensions(['2'], [u'Length (mm)']))

        self.assertFalse(
            validate_dimensions([], [u'Length (mm)']),
            'There has to be the same amount of dimensions.')
        self.assertFalse(
            validate_dimensions([], None),
            'Both parameter have to be specified.')
        self.assertTrue(validate_dimensions(['2.2'], [u'Length (mm)']))
        self.assertFalse(
            validate_dimensions([-1], [u'Length (mm)']),
            'Negative dimensions are not allowed.')

    def test_dimensions_format(self):
        # this tests check if all selectable dimension configs are valid
        for key, config in selectable_dimensions.iteritems():
            self.assertIn('label', config)
            self.assertIn('dimensions', config)
            self.assertIn('dimension_unit', config)
            self.assertTrue(isinstance(config['dimensions'], list))

            if key == 'no_dimensions':
                continue

            if 'price_unit' in config:
                self.assertIn('price_to_dimension_modifier', config)

    @browsing
    def test_different_price_unit(self, browser):
        """ This tests creates a shopitem which takes g as input and displays
            the price in kg. The final price has to convert between those units.
        """
        self.priceitem = create(Builder('shop item')
                                .titled('Farbe')
                                .having(price='200.00',
                                        showPrice=True,
                                        selectable_dimensions='weight_g_kg')
                                .within(self.category))

        browser.login().visit(self.priceitem)

        dim = browser.css('.dimensions-selection input[name="dimension:float"]')
        dim[0].value = '150'
        browser.fill({'quantity:int': '1'}).submit()

        browser.open(self.portal, view='cart_edit')

        self.assertEquals('30.00', browser.css('form > div b span').first.text)



from decimal import Decimal
from ftw.shop.tests.base import FtwShopTestCase
from ftw.shop.utils import to_decimal
from ftw.shop.utils import is_email_valid
from unittest2 import TestCase


class TestUtils(FtwShopTestCase):

    def afterSetUp(self):
        super(TestUtils, self).afterSetUp()

    def test_to_decimal(self):
        self.assertEquals(to_decimal('10.00'), Decimal('10.00'))
        self.assertEquals(to_decimal('123'), Decimal('123.00'))
        self.assertEquals(to_decimal('50.25999'), Decimal('50.25'))
        self.assertEquals(to_decimal('NaN'), 'NaN')
        self.assertEquals(to_decimal('invalid'), 'invalid')

        self.assertEquals(to_decimal(0), Decimal('0.00'))
        self.assertEquals(to_decimal(Decimal('0E-10')), Decimal('0.00'))


class TestIsEmailValid(TestCase):

    def test_valid_addresses(self):
        self.assert_valid_email('john@doe.com')
        self.assert_valid_email('john.doe@gmail.com')
        self.assert_valid_email('j@doe.com')
        self.assert_valid_email('j.d@doe.com')
        self.assert_valid_email('me@john.doe.com')
        self.assert_valid_email('x@y.z')

    def test_invalid_addresses(self):
        self.assert_invalid_email('john@doe@gmail.com')
        self.assert_invalid_email('john.@gmail.com')
        self.assert_invalid_email('john@.gmail.com')
        self.assert_invalid_email('john@localhost')

    def assert_valid_email(self, address, msg=''):
        message = 'Expected email {0} to be valid'.format(address)
        if msg:
            message = ': '.join((message, msg))
        self.assertTrue(is_email_valid(address),
                        message)

    def assert_invalid_email(self, address, msg=''):
        message = 'Expected email {0} to be invalid'.format(address)
        if msg:
            message = ': '.join((message, msg))
        self.assertFalse(is_email_valid(address),
                         message)



import unittest
from zope.schema.vocabulary import SimpleTerm

from ftw.shop.tests.base import FtwShopTestCase
from ftw.shop.vocabularies import create_terms_from_adapters
from ftw.shop.vocabularies import ContactInfoStepGroups
from ftw.shop.vocabularies import OrderReviewStepGroups
from ftw.shop.vocabularies import PaymentProcessorStepGroups
from ftw.shop.vocabularies import PaymentProcessors
from ftw.shop.vocabularies import EnabledPaymentProcessors
from ftw.shop.vocabularies import OrderStorageVocabulary

class TestVocabularies(FtwShopTestCase):
    
    def afterSetUp(self):
        super(TestVocabularies, self).afterSetUp()

    def test_create_terms_from_adapters(self):
        
        def mock_adapter_generator(n):
            
            class MockAdapter(object):
                pass
            
            for i in range(n):
                mock_adapter = MockAdapter()
                mock_adapter.title = "Adapter Title %s" % i
                yield ("adapter_name_%s" % i, mock_adapter)
    
        mock_adapters = mock_adapter_generator(3)            
        terms = create_terms_from_adapters(mock_adapters)
        expected_terms = [SimpleTerm(value="adapter_name_0", 
                                     token="adapter_name_0", 
                                     title="Adapter Title 0"),
                          SimpleTerm(value="adapter_name_1", 
                                     token="adapter_name_1", 
                                     title="Adapter Title 1"),
                          SimpleTerm(value="adapter_name_2", 
                                     token="adapter_name_2", 
                                     title="Adapter Title 2")]

        for i, term in enumerate(terms):
            self.assertEquals(term.value, expected_terms[i].value)
            self.assertEquals(term.token, expected_terms[i].token)
            self.assertEquals(term.title, expected_terms[i].title)

    def test_contact_info_step_groups(self):
        vocabulary = ContactInfoStepGroups(self.portal)
        self.assertEquals(vocabulary._terms[0].value,
                          u'ftw.shop.DefaultContactInformationStepGroup')
        self.assertEquals(vocabulary._terms[0].token,
                          u'ftw.shop.DefaultContactInformationStepGroup')
        self.assertEquals(vocabulary._terms[0].title,
                          u'title_default_contact_info_step_group')

    def test_order_review_step_groups(self):
        vocabulary = OrderReviewStepGroups(self.portal)
        self.assertEquals(vocabulary._terms[0].value,
                          u'ftw.shop.DefaultOrderReviewStepGroup')
        self.assertEquals(vocabulary._terms[0].token,
                          u'ftw.shop.DefaultOrderReviewStepGroup')
        self.assertEquals(vocabulary._terms[0].title,
                          u'title_default_order_review_step_group')

    def test_payment_processor_step_groups(self):
        vocabulary = PaymentProcessorStepGroups(self.portal)
        self.assertEquals(vocabulary._terms[0].value,
                          u'ftw.shop.DefaultPaymentProcessorStepGroup')
        self.assertEquals(vocabulary._terms[0].token,
                          u'ftw.shop.DefaultPaymentProcessorStepGroup')
        self.assertEquals(vocabulary._terms[0].title,
                          u'title_default_payment_processor_step_group')

    def test_payment_processors(self):
        vocabulary = PaymentProcessors(self.portal)
        self.assertTrue(u'ftw.shop.InvoicePaymentProcessor' in
                        [v for v in [t.value for t in vocabulary._terms]])

    def test_enabled_payment_processors(self):
        vocabulary = EnabledPaymentProcessors(self.portal)
        self.assertEquals(len(vocabulary), 1)
        self.assertEquals(vocabulary._terms[0].value,
                          u'ftw.shop.InvoicePaymentProcessor')
        self.assertEquals(vocabulary._terms[0].token,
                          u'ftw.shop.InvoicePaymentProcessor')
        self.assertEquals(vocabulary._terms[0].title,
                          'Gegen Rechnung')

    def test_order_storage_vocabulary(self):
        vocabulary = OrderStorageVocabulary(self.portal)
        self.assertEquals(vocabulary._terms[0].value,
                          u'ftw.shop.BTreeOrderStorage')
        self.assertEquals(vocabulary._terms[0].token,
                          u'ftw.shop.BTreeOrderStorage')
        self.assertEquals(vocabulary._terms[0].title,
                          u'BTree Storage')


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestVocabularies))
    return suite



"""Payment Processor Widget Implementation
"""
__docformat__ = "reStructuredText"

import zope.component
import zope.interface
import zope.schema.interfaces
from zope.i18n import translate
from zope.component import getAdapters

from z3c.form import interfaces
from z3c.form.widget import FieldWidget
from z3c.form.browser import widget
from z3c.form.browser.radio import RadioWidget

from ftw.shop.interfaces import IPaymentProcessorWidget
from ftw.shop.interfaces import IPaymentProcessor


class PaymentProcessorWidget(RadioWidget):
    """Input type radio payment processor widget implementation."""
    zope.interface.implementsOnly(IPaymentProcessorWidget)

    klass = u'paymentprocessor-widget'
    items = ()

    def update(self):
        """See z3c.form.interfaces.IWidget."""
        super(PaymentProcessorWidget, self).update()
        widget.addFieldClass(self)
        self.items = []

        pp_list = list(getAdapters((self.form.context, self.request, None),
                                   IPaymentProcessor))
        pp_list = filter(lambda pp: pp[1].available(), pp_list)
        available_payment_processors = dict(pp_list)

        for count, term in enumerate(self.terms):
            if not term.value in available_payment_processors:
                continue
            checked = self.isChecked(term)
            id = '%s-%i' % (self.id, count)
            label = term.token
            if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
                label = translate(term.title, context=self.request,
                                  default=term.title)
            self.items.append(
                {'id': id, 'name': self.name + ':list', 'value': term.token,
                 'label': label, 'checked': checked,
                 'image': available_payment_processors[term.token].image,
                 'description': available_payment_processors[term.token].description})


@zope.component.adapter(zope.schema.interfaces.IField, interfaces.IFormLayer)
@zope.interface.implementer(interfaces.IFieldWidget)
def PaymentProcessorFieldWidget(field, request):
    """IFieldWidget factory for PaymentProcessorWidget."""
    return FieldWidget(field, PaymentProcessorWidget(request))



import unittest

from zope.component import getMultiAdapter

from ftw.shop.tests.base import FtwShopTestCase
from pyquery import PyQuery as pq


class TestCategoryView(FtwShopTestCase):

    def afterSetUp(self):
        super(TestCategoryView, self).afterSetUp()

        self.category_view = getMultiAdapter((self.portal.shop.products,
                                 self.portal.REQUEST),
                                 name='view')

    def test_get_items(self):
        # Should be sorted aphabetically by Title
        self.assertEquals(self.category_view.getItems(),
                          [self.movie, self.tshirt, self.book])

    def test_single_item(self):
        item_datas = self.category_view.getItemDatas()
        item = item_datas[0]
        item_listing = self.category_view.single_item(item)

        pq_doc = pq(item_listing)
        self.assertEquals(len(pq_doc("input[name=skuCode][value='12345']")), 1)

    def test_one_variation(self):
        item_datas = self.category_view.getItemDatas()
        item = item_datas[2]
        item_listing = self.category_view.one_variation(item)
        self.assertTrue('Paperback' in item_listing)
        self.assertTrue('Hardcover' in item_listing)

        pq_doc = pq(item_listing)
        self.assertEquals(pq_doc("#var-Hardcover td")[2].text, 'b11')
        self.assertEquals(pq_doc("#var-Paperback td")[2].text, 'b22')

    def test_two_variations(self):
        item_datas = self.category_view.getItemDatas()
        item = item_datas[1]
        item_listing = self.category_view.two_variations(item)

        pq_doc = pq(item_listing)
        self.assertEquals(len(pq_doc('select[name=var1choice] option')), 3)
        self.assertEquals(len(pq_doc('select[name=var2choice] option')), 3)

    def test_get_item_datas(self):
        item_datas = self.category_view.getItemDatas()
        movie_data = item_datas[0]
        tshirt_data = item_datas[1]
        book_data = item_datas[2]

        self.assertEquals(movie_data['description'],
                          'A Shop Item with no variations')
        self.assertEquals(movie_data['hasVariations'], False)
        self.assertEquals(movie_data['imageTag'], None)
        self.assertEquals(movie_data['skuCode'], '12345')
        self.assertEquals(movie_data['price'], '7.15')
        self.assertEquals(movie_data['title'], 'A Movie')
        self.assertEquals(movie_data['varConf'], None)
        self.assertEquals(movie_data['variants'], None)

        self.assertEquals(book_data['description'],
                          'A Shop Item with one variation')
        self.assertEquals(book_data['hasVariations'], True)
        self.assertEquals(book_data['imageTag'], None)
        self.assertEquals(book_data['skuCode'], None)
        self.assertEquals(book_data['price'], None)
        self.assertEquals(book_data['title'], 'Professional Plone Development')
        self.assertTrue(book_data['varConf'])
        self.assertEquals(book_data['variants'], None)

        self.assertEquals(tshirt_data['description'],
                          'A Shop Item with two variations')
        self.assertEquals(tshirt_data['hasVariations'], True)
        self.assertEquals(tshirt_data['imageTag'], None)
        self.assertEquals(tshirt_data['skuCode'], None)
        self.assertEquals(tshirt_data['price'], None)
        self.assertEquals(tshirt_data['title'], 'A T-Shirt')
        self.assertTrue(tshirt_data['varConf'])
        self.assertEquals(tshirt_data['variants'], None)


    def test_categories(self):
        categories = self.category_view.categories
        self.assertEquals(len(categories), 1)
        self.assertTrue(self.subcategory.id in categories[0]['url'])

    def test_category_contents(self):
        category_contents = self.category_view.category_contents
        self.assertEquals(len(category_contents), 4)
        self.assertTrue(self.movie in category_contents)
        self.assertTrue(self.book in category_contents)
        self.assertTrue(self.tshirt in category_contents)
        self.assertTrue(self.subcategory in category_contents)

    def test_category_contents_ordering(self):
        self.tshirt.setRankForCategory(self.portal.shop.products, 10)
        self.movie.setRankForCategory(self.portal.shop.products, 20)
        self.book.setRankForCategory(self.portal.shop.products, 30)
        self.subcategory.setRankForCategory(self.portal.shop.products, 40)

        category_contents = self.category_view.category_contents
        self.assertEquals(category_contents,
                          [self.tshirt,
                           self.movie,
                           self.book,
                           self.subcategory])


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestCategoryView))
    return suite
